@@ -1,5 +1,5 @@
 /*
-  $Id: sample6.c,v 1.4 2004/02/04 10:22:01 rocky Exp $
+  $Id: sample6.c,v 1.7 2004/03/21 00:57:03 rocky Exp $
 
   Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
   
@@ -21,6 +21,11 @@
 /* Simple program to show using libiso9660 to extract a file from a
    cue/bin CD-IMAGE.
  */
+
+#if defined ( WIN32 )
+#define ftruncate chsize
+#endif
+
 /* This is the CD-image with an ISO-9660 filesystem */
 #define ISO9660_IMAGE_PATH "../"
 #define ISO9660_IMAGE ISO9660_IMAGE_PATH "test/isofs-m1.cue"
@@ -31,15 +36,38 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-#include <sys/types.h>
+
 #include <cdio/cdio.h>
 #include <cdio/iso9660.h>
 
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
-#include <unistd.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STDIO_H
+#include <stdio.h>
+#endif
+#ifdef HAVE_ERRNO_H
+#include <errno.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#define my_exit(rc)				\
+  fclose (outfd);				\
+  free(statbuf);				\
+  cdio_destroy(cdio);				\
+  return rc;					\
+  
 
 int
 main(int argc, const char *argv[])
@@ -62,12 +90,15 @@ main(int argc, const char *argv[])
       fprintf(stderr, 
 	      "Could not get ISO-9660 file information for file %s\n",
 	      ISO9660_FILENAME);
+      cdio_destroy(cdio);
       return 2;
     }
 
   if (!(outfd = fopen ("copying", "wb")))
     {
       perror ("fopen()");
+      cdio_destroy(cdio);
+      free(statbuf);
       return 3;
     }
 
@@ -84,7 +115,7 @@ main(int argc, const char *argv[])
       {
 	fprintf(stderr, "Error reading ISO 9660 file at lsn %lu\n",
 		(long unsigned int) statbuf->lsn + (i / ISO_BLOCKSIZE));
-	return 4;
+	my_exit(4);
       }
       
       
@@ -93,7 +124,7 @@ main(int argc, const char *argv[])
       if (ferror (outfd))
 	{
 	  perror ("fwrite()");
-	  return 5;
+	  my_exit(5);
 	}
     }
   
@@ -105,7 +136,5 @@ main(int argc, const char *argv[])
   if (ftruncate (fileno (outfd), statbuf->size))
     perror ("ftruncate()");
 
-  fclose (outfd);
-  cdio_destroy(cdio);
-  return 0;
+  my_exit(0);
 }
