@@ -1,5 +1,5 @@
 /*
-    $Id: iso9660.h,v 1.39 2004/02/26 00:13:24 rocky Exp $
+    $Id: iso9660.h,v 1.44 2004/06/19 02:27:19 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -61,6 +61,24 @@
 
 #define MAX_ISOPATHNAME 255
 
+/*! \def Max # characters in an perparer id. */
+#define ISO_MAX_PREPARER_ID 128
+
+/*! \def Max # characters in an publisher id. */
+#define ISO_MAX_PUBLISHER_ID 128
+
+/*! \def Max # characters in an application id. */
+#define ISO_MAX_APPLICATION_ID 128
+
+/*! \def Max # characters in an system id. */
+#define ISO_MAX_SYSTEM_ID 32
+
+/*! \def Max # characters in an volume id. */
+#define ISO_MAX_VOLUME_ID 32
+
+/*! \def Max # characters in an volumeset id. */
+#define ISO_MAX_VOLUMESET_ID 128
+
 /*
  * ISO 9660 directory flags.
  */
@@ -148,29 +166,29 @@ typedef struct iso9660_ltime  iso9660_ltime_t;
   \brief ISO-9660 Primary Volume Descriptor.
  */
 struct iso9660_pvd {
-  uint8_t          type;                      /**< 711 encoded */
+  uint8_t          type;                         /**< 711 encoded */
   char             id[5];
-  uint8_t          version;                   /**< 711 encoded */
+  uint8_t          version;                      /**< 711 encoded */
   char             unused1[1];
-  char             system_id[32];             /**< each char is an achar */
-  char             volume_id[32];             /**< each char is a dchar */
+  char             system_id[ISO_MAX_SYSTEM_ID]; /**< each char is an achar */
+  char             volume_id[ISO_MAX_VOLUME_ID]; /**< each char is a dchar */
   char             unused2[8];
-  uint64_t         volume_space_size;         /**< 733 encoded */
+  uint64_t         volume_space_size;            /**< 733 encoded */
   char             escape_sequences[32];
-  uint32_t         volume_set_size;           /**< 723 encoded */
-  uint32_t         volume_sequence_number;    /**< 723 encoded */
-  uint32_t         logical_block_size;        /**< 723 encoded */
-  uint64_t         path_table_size;           /**< 733 encoded */
-  uint32_t         type_l_path_table;         /**< 731 encoded */
-  uint32_t         opt_type_l_path_table;     /**< 731 encoded */
-  uint32_t         type_m_path_table;         /**< 732 encoded */
-  uint32_t         opt_type_m_path_table;     /**< 732 encoded */
-  char             root_directory_record[34]; /**< See section 9.1 of
-                                                 ISO 9660 spec. */
-  char             volume_set_id[128];        /**< dchars */
-  char             publisher_id[128];         /**< achars */
-  char             preparer_id[128];          /**< achars */
-  char             application_id[128];       /**< achars */
+  uint32_t         volume_set_size;              /**< 723 encoded */
+  uint32_t         volume_sequence_number;       /**< 723 encoded */
+  uint32_t         logical_block_size;           /**< 723 encoded */
+  uint64_t         path_table_size;              /**< 733 encoded */
+  uint32_t         type_l_path_table;            /**< 731 encoded */
+  uint32_t         opt_type_l_path_table;        /**< 731 encoded */
+  uint32_t         type_m_path_table;            /**< 732 encoded */
+  uint32_t         opt_type_m_path_table;        /**< 732 encoded */
+  char             root_directory_record[34];    /**< See section 9.1 of
+                                                    ISO 9660 spec. */
+  char             volume_set_id[ISO_MAX_VOLUMESET_ID];    /**< dchars */
+  char             publisher_id[ISO_MAX_PUBLISHER_ID];     /**< achars */
+  char             preparer_id[ISO_MAX_PREPARER_ID];       /**< achars */
+  char             application_id[ISO_MAX_APPLICATION_ID]; /**< achars */
   char             copyright_file_id[37];     /**< See section 7.5 of
                                                  ISO 9660 spec. Each char is 
                                                  a dchar */
@@ -266,14 +284,29 @@ typedef struct _iso9660 iso9660_t;
   True is unconditionally returned. If there was an error false would
   be returned.
 */
-  bool iso9660_close (iso9660_t * iso);
+  bool iso9660_close (iso9660_t * p_iso);
 
 
 /*!
   Seek to a position and then read n bytes. Size read is returned.
 */
-  long int iso9660_iso_seek_read (iso9660_t *iso, void *ptr, lsn_t start, 
-                                  long int size);
+  long int iso9660_iso_seek_read (const iso9660_t *p_iso, void *ptr, 
+                                  lsn_t start, long int size);
+
+/*!
+  Read the Primary Volume Descriptor for an ISO 9660 image.
+  True is returned if read, and false if there was an error.
+*/
+  bool iso9660_ifs_read_pvd (const iso9660_t *p_iso, 
+                             /*out*/ iso9660_pvd_t *p_pvd);
+
+/*!
+  Read the Primary Volume Descriptor for a CD.
+  True is returned if read, and false if there was an error.
+*/
+  bool iso9660_fs_read_mode2_pvd (const CdIo *p_cdio, 
+                                  /*out*/ iso9660_pvd_t *p_pvd,
+                                  bool b_form2);
 
 /*====================================================
   Time conversion 
@@ -289,7 +322,7 @@ typedef struct _iso9660 iso9660_t;
   Set "long" time in format used in ISO 9660 primary volume descriptor
   from a Unix time structure. */
   void iso9660_set_ltime (const struct tm *_tm, 
-                          /*out*/ iso9660_ltime_t *pvd_date);
+                          /*out*/ iso9660_ltime_t *p_pvd_date);
 
 /*!
   Get Unix time structure from format use in an ISO 9660 directory index 
@@ -328,7 +361,7 @@ bool iso9660_isachar (int c);
 
    The length of the translated string is returned.
 */
-int iso9660_name_translate(const char *old, char *new);
+int iso9660_name_translate(const char *oldname, char *newname);
 
 /*!  
   Pad string src with spaces to size len and copy this to dst. If
@@ -412,7 +445,7 @@ iso9660_dir_calc_record_size (unsigned int namelen, unsigned int su_len);
 
    Returns stat_t of entry if we found lsn, or NULL otherwise.
  */
-iso9660_stat_t *iso9660_find_fs_lsn(const CdIo *cdio, lsn_t lsn);
+iso9660_stat_t *iso9660_find_fs_lsn(const CdIo *p_cdio, lsn_t lsn);
 
 
 /*!
@@ -468,6 +501,13 @@ void * iso9660_fs_readdir (const CdIo *obj, const char pathname[], bool mode2);
 */
 void * iso9660_ifs_readdir (iso9660_t *iso, const char pathname[]);
 
+/*!
+  Return the PVD's application ID.
+  NULL is returned if there is some problem in getting this. 
+  */
+const char * iso9660_get_application_id(const iso9660_pvd_t *p_pvd);
+
+
 uint8_t iso9660_get_dir_len(const iso9660_dir_t *idr);
 
 #if FIXME
@@ -483,6 +523,18 @@ lsn_t iso9660_get_dir_extent(const iso9660_dir_t *idr);
 */
 char * iso9660_dir_to_name (const iso9660_dir_t *iso9660_dir);
   
+/*!
+   Return a string containing the preparer id with trailing
+   blanks removed.
+*/
+const char *iso9660_get_preparer_id(const iso9660_pvd_t *pvd);
+  
+/*!
+   Return a string containing the PVD's publisher id with trailing
+   blanks removed.
+*/
+const char *iso9660_get_publisher_id(const iso9660_pvd_t *pvd);
+
 uint8_t iso9660_get_pvd_type(const iso9660_pvd_t *pvd);
 
 const char * iso9660_get_pvd_id(const iso9660_pvd_t *pvd);
@@ -496,11 +548,29 @@ int iso9660_get_pvd_block_size(const iso9660_pvd_t *pvd) ;
  */
 int iso9660_get_pvd_version(const iso9660_pvd_t *pvd) ;
 
+/*!
+   Return a string containing the PVD's system id with trailing
+   blanks removed.
+*/
+const char *iso9660_get_system_id(const iso9660_pvd_t *pvd);
+  
+
 /*! Return the LSN of the root directory for pvd.
     If there is an error CDIO_INVALID_LSN is returned. 
  */
 lsn_t iso9660_get_root_lsn(const iso9660_pvd_t *pvd);
 
+/*!
+  Return the PVD's volume ID.
+*/
+const char *iso9660_get_volume_id(const iso9660_pvd_t *pvd);
+
+/*!
+  Return the PVD's volumeset ID.
+  NULL is returned if there is some problem in getting this. 
+*/
+const char *iso9660_get_volumeset_id(const iso9660_pvd_t *pvd);
+
 /* pathtable */
 
 /*! Zero's out pathable. Do this first. */
