@@ -1,8 +1,8 @@
 /*
-    $Id: sector.h,v 1.8 2003/11/17 12:06:58 rocky Exp $
+    $Id: sector.h,v 1.13 2004/06/02 00:43:53 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@
 
 #include <cdio/types.h>
 
-#define CDIO_PREGAP_SECTORS 150
+#define CDIO_PREGAP_SECTORS  150
 #define CDIO_POSTGAP_SECTORS 150
 
 /* 
@@ -75,25 +75,28 @@
    This is from linux.h - not to slight other OS's. This was the first
    place I came across such useful stuff.
 */
-#define CDIO_CD_MINS           74 /**< max. minutes per CD, not really
-                                       a limit */
-#define CDIO_CD_SECS_PER_MIN   60 /**< seconds per minute */
-#define CDIO_CD_FRAMES_PER_SEC 75 /**< frames per second */
-#define CDIO_CD_SYNC_SIZE      12 /**< 12 sync bytes per raw data frame */
-#define CDIO_CD_CHUNK_SIZE     24 /**< lowest-level "data bytes piece" */
-#define CDIO_CD_NUM_OF_CHUNKS  98 /**< chunks per frame */
-#define CDIO_CD_FRAMESIZE_SUB  96 /**< subchannel data "frame" size */
-#define CDIO_CD_HEADER_SIZE     4 /**< header (address) bytes per raw
-                                     data frame */
-#define CDIO_CD_SUBHEADER_SIZE  8 /**< subheader bytes per raw XA data frame */
-#define CDIO_CD_EDC_SIZE        4 /**< bytes EDC per most raw data
-                                     frame types */
-#define CDIO_CD_M1F1_ZERO_SIZE  8 /**< bytes zero per yellow book mode
-                                     1 frame */
-#define CDIO_CD_ECC_SIZE      276 /**< bytes ECC per most raw data frame types */
-#define CDIO_CD_FRAMESIZE    2048 /**< bytes per frame, "cooked" mode */
-#define CDIO_CD_FRAMESIZE_RAW 2352/**< bytes per frame, "raw" mode */
-#define CDIO_CD_FRAMESIZE_RAWER 2646 /**< The maximum possible returned bytes */ 
+#define CDIO_CD_MINS              74   /**< max. minutes per CD, not really
+                                         a limit */
+#define CDIO_CD_SECS_PER_MIN      60   /**< seconds per minute */
+#define CDIO_CD_FRAMES_PER_SEC    75   /**< frames per second */
+#define CDIO_CD_SYNC_SIZE         12   /**< 12 sync bytes per raw data frame */
+#define CDIO_CD_CHUNK_SIZE        24   /**< lowest-level "data bytes piece" */
+#define CDIO_CD_NUM_OF_CHUNKS     98   /**< chunks per frame */
+#define CDIO_CD_FRAMESIZE_SUB     96   /**< subchannel data "frame" size */
+#define CDIO_CD_HEADER_SIZE        4   /**< header (address) bytes per raw
+                                          data frame */
+#define CDIO_CD_SUBHEADER_SIZE     8   /**< subheader bytes per raw XA data 
+                                            frame */
+#define CDIO_CD_EDC_SIZE           4   /**< bytes EDC per most raw data
+                                          frame types */
+#define CDIO_CD_M1F1_ZERO_SIZE     8   /**< bytes zero per yellow book mode
+                                          1 frame */
+#define CDIO_CD_ECC_SIZE         276   /**< bytes ECC per most raw data frame 
+                                          types */
+#define CDIO_CD_FRAMESIZE       2048   /**< bytes per frame, "cooked" mode */
+#define CDIO_CD_FRAMESIZE_RAW   2352   /**< bytes per frame, "raw" mode */
+#define CDIO_CD_FRAMESIZE_RAWER 2646   /**< The maximum possible returned 
+                                          bytes */ 
 #define CDIO_CD_FRAMESIZE_RAW1 (CDIO_CD_CD_FRAMESIZE_RAW-CDIO_CD_SYNC_SIZE) /*2340*/
 #define CDIO_CD_FRAMESIZE_RAW0 (CDIO_CD_FRAMESIZE_RAW-CDIO_CD_SYNC_SIZE-CDIO_CD__HEAD_SIZE) /*2336*/
 
@@ -106,12 +109,15 @@
 /*! "before data" sync bytes + header of XA (green, mode2) frame */
 #define CDIO_CD_XA_SYNC_HEADER   (CDIO_CD_SYNC_SIZE+CDIO_CD_XA_HEADER) 
 
-/* CD-ROM address types (Linux cdrom_tocentry.cdte_format) */
+/*! CD-ROM address types (GNU/Linux e.g. cdrom_tocentry.cdte_format) */
 #define	CDIO_CDROM_LBA 0x01 /**< "logical block": first frame is #0 */
 #define	CDIO_CDROM_MSF 0x02 /**< "minute-second-frame": binary, not
                                BCD here! */
 
+/*! CD-ROM track format types (GNU/Linux cdte_ctrl) */
 #define	CDIO_CDROM_DATA_TRACK	0x04
+#define	CDIO_CDROM_CDI_TRACK	0x10
+#define	CDIO_CDROM_XA_TRACK	0x20
 
 /*! The leadout track is always 0xAA, regardless of # of tracks on disc */
 #define	CDIO_CDROM_LEADOUT_TRACK 0xAA
@@ -120,8 +126,11 @@
 #define M2SUB_SECTOR_SIZE   2332
 #define M2RAW_SECTOR_SIZE   2336
 
-#define CDIO_CD_MAX_TRACKS 99 
-#define CDIO_CD_MIN_TRACK_NO 1
+#define CDIO_CD_MAX_TRACKS    99 
+#define CDIO_CD_MIN_TRACK_NO   1
+#define CDIO_CD_MAX_LSN   450150
+#define CDIO_CD_MIN_LSN  -450150
+
 
 #define CDIO_CD_FRAMES_PER_MIN \
    (CDIO_CD_FRAMES_PER_SEC*CDIO_CD_SECS_PER_MIN)
@@ -140,6 +149,11 @@
   \warning cdio_lba_to_msf_str returns new allocated string */
 char *cdio_lba_to_msf_str (lba_t lba);
 
+/*! 
+  Convert an MSF into a string representation of the MSF.
+  \warning cdio_msf_to_msf_str returns new allocated string */
+char *cdio_msf_to_str (const msf_t *msf);
+
 /*! 
   Convert an LBA into the corresponding LSN.
 */
