@@ -1,5 +1,5 @@
 /*
-    $Id: types.h,v 1.10 2004/03/13 03:32:49 rocky Exp $
+    $Id: types.h,v 1.17 2004/05/26 06:29:15 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -204,14 +204,19 @@ extern "C" {
   typedef uint8_t bitfield_t;
 #endif
   
-  /*! The type of a Logical Block Address. 
+  /*! The type of a Logical Block Address. We allow for an lba to be 
+    negative to be consistent with an lba, although I'm not sure this
+    this is possible.
+      
+   */
+  typedef int32_t lba_t;
+  
+  /*! The type of a Logical Sector Number. Note that an lba lsn be negative
+    and the MMC3 specs allow for a conversion of a negative lba
 
     @see msf_t
   */
-  typedef uint32_t lba_t;
-  
-  /*! The type of an Logical Sector Number. */
-  typedef uint32_t lsn_t;
+  typedef int32_t lsn_t;
   
   /*! The type of an track number 0..99. */
   typedef uint8_t track_t;
@@ -222,17 +227,93 @@ extern "C" {
 #define CDIO_INVALID_TRACK   0xFF
   
   /*! 
-    Constant for invalid LBA
+    Constant for invalid LBA. It is 151 less than the most negative
+    LBA -45150. This provide slack for the 150-frame offset in
+    LBA to LSN 150 conversions
   */
-#define CDIO_INVALID_LBA   0xFFFFFFFF
+#define CDIO_INVALID_LBA    -45301
   
   /*! 
     Constant for invalid LSN
   */
-#define CDIO_INVALID_LSN   0xFFFFFFFF
+#define CDIO_INVALID_LSN    CDIO_INVALID_LBA
+
+  /*! 
+    Number of ASCII bytes in a media catalog number (MCN).
+  */
+#define CDIO_MCN_SIZE       13
 
 typedef int cdio_fs_anal_t;
 
+/*! The type of an drive capability bit mask. See below for values*/
+  typedef uint32_t cdio_drive_cap_t;
+  
+/*!
+  \brief Drive types returned by cdio_get_drive_cap()
+
+  Most are copied from the GNU/Linux the uniform CD-ROM driver header
+  linux/cdrom.h>  NOTE: Setting a bit here means the presence of
+  a capability.
+*/ 
+
+#define CDIO_DRIVE_CAP_CLOSE_TRAY     0x00001 /**< caddy systems can't 
+                                                   close... */
+#define CDIO_DRIVE_CAP_OPEN_TRAY      0x00002 /**< but can eject.  */
+#define CDIO_DRIVE_CAP_LOCK	      0x00004 /**< disable manual eject */
+#define CDIO_DRIVE_CAP_SELECT_SPEED   0x00008 /**< programmable speed */
+#define CDIO_DRIVE_CAP_SELECT_DISC    0x00010 /**< select disc from juke-box */
+#define CDIO_DRIVE_CAP_MULTI_SESSION  0x00020 /**< read sessions>1 */
+#define CDIO_DRIVE_CAP_MCN	      0x00040 /**< Medium Catalog Number */
+#define CDIO_DRIVE_CAP_MEDIA_CHANGED  0x00080 /**< media changed */
+#define CDIO_DRIVE_CAP_CD_AUDIO	      0x00100 /**< drive can play CD audio */
+#define CDIO_DRIVE_CAP_RESET          0x00200 /**< hard reset device */
+#define CDIO_DRIVE_CAP_IOCTLS         0x00400 /**< driver has non-standard 
+                                                   ioctls */
+#define CDIO_DRIVE_CAP_DRIVE_STATUS   0x00800 /**< driver implements drive 
+                                               status */
+#define CDIO_DRIVE_CAP_GENERIC_PACKET 0x01000 /**< driver implements generic 
+                                                   packets */
+#define CDIO_DRIVE_CAP_CD_R	      0x02000 /**< drive can write CD-R */
+#define CDIO_DRIVE_CAP_CD_RW	      0x04000 /**< drive can read CD-RW */
+#define CDIO_DRIVE_CAP_DVD	      0x08000 /**< drive can read DVD */
+#define CDIO_DRIVE_CAP_DVD_R	      0x10000 /**< drive can write DVD-R */
+#define CDIO_DRIVE_CAP_DVD_RAM	      0x20000 /**< drive can write DVD-RAM */
+
+/**<  These are not taken by GNU/Linux cdrom.h (yet) */
+#define CDIO_DRIVE_CAP_ERROR          0x00000 /**< Error */
+#define CDIO_DRIVE_CAP_FILE	      0x40000 /**< drive is really a file, i.e
+                                                   a CD file image */
+#define CDIO_DRIVE_CAP_UNKNOWN        0x80000 /**< Dunno. It can be on if we
+					        have only partial information 
+                                                or are not completely certain
+                                          */
+/**< Masks derived from above... */
+#define CDIO_DRIVE_CAP_CD_WRITER \
+   (CDIO_DRIVE_CAP_CD_R|CDIO_DRIVE_CAP_CD_RW) 
+/**< Has some sort of CD writer ability */
+
+#define CDIO_DRIVE_CAP_CD \
+   (CDIO_DRIVE_CAP_CD_AUDIO|CDIO_DRIVE_CAP_CD_WRITER)
+/**< Has some sort of CD ability */
+
+#define CDIO_DRIVE_CAP_DVD_WRITER \
+   (CDIO_DRIVE_CAP_DVD_R|CDIO_DRIVE_CAP_DVD_RAM)
+/**< Has some sort of DVD writer ability */
+
+/*! 
+  track flags
+  Q Sub-channel Control Field (4.2.3.3)
+*/
+typedef enum {
+  CDIO_TRACK_FLAG_NONE = 		0x00,	/**< no flags set */
+  CDIO_TRACK_FLAG_PRE_EMPHASIS =	0x01,	/**< audio track recorded with
+                                                     pre-emphasis */
+  CDIO_TRACK_FLAG_COPY_PERMITTED =	0x02,	/**< digital copy permitted */
+  CDIO_TRACK_FLAG_DATA =		0x04,	/**< data track */
+  CDIO_TRACK_FLAG_FOUR_CHANNEL_AUDIO =	0x08,	/**< 4 audio channels */
+  CDIO_TRACK_FLAG_SCMS =			0x10	/**< SCMS (5.29.2.7) */
+} cdio_track_flag;
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
