@@ -1,5 +1,5 @@
 /*
-    $Id: win32.c,v 1.4 2004/03/07 02:40:58 rocky Exp $
+    $Id: win32.c,v 1.17 2004/06/20 15:06:42 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
 
@@ -26,14 +26,14 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: win32.c,v 1.4 2004/03/07 02:40:58 rocky Exp $";
+static const char _rcsid[] = "$Id: win32.c,v 1.17 2004/06/20 15:06:42 rocky Exp $";
 
 #include <cdio/cdio.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
+#include <cdio/scsi_mmc.h>
 #include "cdio_assert.h"
 #include "cdio_private.h" /* protoype for cdio_is_device_win32 */
-#include "scsi_mmc.h"
 
 #include <string.h>
 
@@ -72,12 +72,51 @@ _cdio_mciSendCommand(int id, UINT msg, DWORD flags, void *arg)
   return(mci_error == 0);
 }
 
+static access_mode_t 
+str_to_access_mode_win32(const char *psz_access_mode) 
+{
+  const access_mode_t default_access_mode = 
+    WIN_NT ? _AM_IOCTL : _AM_ASPI;
+
+  if (NULL==psz_access_mode) return default_access_mode;
+  
+  if (!strcmp(psz_access_mode, "ioctl"))
+    return _AM_IOCTL;
+  else if (!strcmp(psz_access_mode, "ASPI"))
+    return _AM_ASPI;
+  else {
+    cdio_warn ("unknown access type: %s. Default used.", 
+	       psz_access_mode);
+    return default_access_mode;
+  }
+}
+
 static const char *
 cdio_is_cdrom(const char drive_letter) {
   if ( WIN_NT ) {
-    return win32ioctl_is_cdrom(drive_letter);
+    return is_cdrom_win32ioctl (drive_letter);
   } else {
-    return wnaspi32_is_cdrom(drive_letter);
+    return is_cdrom_aspi(drive_letter);
+  }
+}
+
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+static cdio_drive_cap_t
+_cdio_get_drive_cap (const void *env) {
+  const _img_private_t *_obj = env;
+
+  if (_obj->hASPI) {
+    /* A safe guess */ 
+
+    return get_drive_cap_aspi (env);
+  } else {
+    return get_drive_cap_win32ioctl (env);
   }
 }
 
@@ -99,14 +138,16 @@ _cdio_init_win32 (void *user_data)
 
   /* Initializations */
   env->h_device_handle = NULL;
-  env->i_sid = 0;
-  env->hASPI = 0;
-  env->lpSendCommand = 0;
-  
-  if ( WIN_NT ) {
-    return win32ioctl_init_win32(env);
+  env->i_sid           = 0;
+  env->hASPI           = 0;
+  env->lpSendCommand   = 0;
+  env->b_aspi_init     = false;
+  env->b_ioctl_init    = false;
+
+  if ( _AM_IOCTL == env->access_mode ) {
+    return init_win32ioctl(env);
   } else {
-    return wnaspi32_init_win32(env);
+    return init_aspi(env);
   }
 }
 
@@ -114,7 +155,7 @@ _cdio_init_win32 (void *user_data)
   Release and free resources associated with cd. 
  */
 static void
-_cdio_win32_free (void *user_data)
+_free_win32 (void *user_data)
 {
   _img_private_t *env = user_data;
 
@@ -139,9 +180,9 @@ _cdio_read_audio_sectors (void *user_data, void *data, lsn_t lsn,
 {
   _img_private_t *env = user_data;
   if ( env->hASPI ) {
-    return wnaspi32_read_audio_sectors( env, data, lsn, nblocks );
+    return read_audio_sectors_aspi( env, data, lsn, nblocks );
   } else {
-    return win32ioctl_read_audio_sectors( env, data, lsn, nblocks );
+    return read_audio_sectors_win32ioctl( env, data, lsn, nblocks );
   }
 }
 
@@ -170,9 +211,9 @@ _cdio_read_mode1_sector (void *user_data, void *data, lsn_t lsn,
   env->gen.ioctls_debugged++;
 
   if ( env->hASPI ) {
-    return 1;
+    return read_mode1_sector_aspi( env, data, lsn, b_form2 );
   } else {
-    return win32ioctl_read_mode1_sector( env, data, lsn, b_form2 );
+    return read_mode1_sector_win32ioctl( env, data, lsn, b_form2 );
   }
 }
 
@@ -234,7 +275,7 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
 
   if ( env->hASPI ) {
     int ret;
-    ret = wnaspi32_read_mode2_sector(user_data, buf, lsn);
+    ret = read_mode2_sector_aspi(user_data, buf, lsn, 1);
     if( ret != 0 ) return ret;
     if (b_form2)
       memcpy (data, buf, M2RAW_SECTOR_SIZE);
@@ -242,7 +283,7 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
       memcpy (((char *)data), buf + CDIO_CD_SUBHEADER_SIZE, CDIO_CD_FRAMESIZE);
     return 0;
   } else {
-    return win32ioctl_read_mode2_sector( env, data, lsn, b_form2 );
+    return read_mode2_sector_win32ioctl( env, data, lsn, b_form2 );
   }
 }
 
@@ -283,7 +324,7 @@ _cdio_stat_size (void *user_data)
   Set the key "arg" to "value" in source device.
 */
 static int
-_cdio_set_arg (void *user_data, const char key[], const char value[])
+_set_arg_win32 (void *user_data, const char key[], const char value[])
 {
   _img_private_t *env = user_data;
 
@@ -296,6 +337,17 @@ _cdio_set_arg (void *user_data, const char key[], const char value[])
       
       env->gen.source_name = strdup (value);
     }
+  else if (!strcmp (key, "access-mode"))
+    {
+      env->access_mode = str_to_access_mode_win32(value);
+      if (env->access_mode == _AM_ASPI && !env->b_aspi_init) 
+	return init_aspi(env) ? 1 : -3;
+      else if (env->access_mode == _AM_IOCTL && !env->b_ioctl_init) 
+	return init_win32ioctl(env) ? 1 : -3;
+      else
+	return -4;
+      return 0;
+    }
   else 
     return -1;
 
@@ -311,9 +363,9 @@ _cdio_read_toc (_img_private_t *env)
 {
   bool ret;
   if( env->hASPI ) {
-    ret = wnaspi32_read_toc( env );
+    ret = read_toc_aspi( env );
   } else {
-    ret =win32ioctl_read_toc(env);
+    ret = read_toc_win32ioctl( env );
   }
   if (ret) env->gen.toc_init = true ;
   return true;
@@ -369,10 +421,8 @@ _cdio_get_arg (void *user_data, const char key[])
   } else if (!strcmp (key, "access-mode")) {
     if (env->hASPI) 
       return "ASPI";
-    else if ( WIN_NT ) 
-      return "winNT/2K/XP ioctl";
     else 
-      return "undefined WIN32";
+      return "ioctl";
   } 
   return NULL;
 }
@@ -388,7 +438,7 @@ _cdio_get_first_track_num(void *user_data)
   
   if (!env->toc_init) _cdio_read_toc (env) ;
 
-  return env->first_track_num;
+  return env->i_first_track;
 }
 
 /*!
@@ -399,12 +449,12 @@ _cdio_get_first_track_num(void *user_data)
 
  */
 static char *
-_cdio_get_mcn (void *env) {
+_cdio_get_mcn (const void *env) {
 
-  _img_private_t *_env = env;
+  const _img_private_t *_env = env;
 
   if( ! _env->hASPI ) {
-    win32ioctl_get_mcn(_env);
+    return get_mcn_win32ioctl(_env);
   }
   return NULL;
 }
@@ -437,9 +487,9 @@ _cdio_get_track_format(void *obj, track_t track_num)
     return TRACK_FORMAT_ERROR;
 
   if( env->hASPI ) {
-    return wnaspi32_get_track_format(env, track_num);
+    return get_track_format_aspi(env, track_num);
   } else {
-    return win32ioctl_get_track_format(env, track_num);
+    return get_track_format_win32ioctl(env, track_num);
   }
 }
 
@@ -452,22 +502,27 @@ _cdio_get_track_format(void *obj, track_t track_num)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *obj, track_t track_num) 
+_cdio_get_track_green(void *obj, track_t i_track) 
 {
   _img_private_t *env = obj;
   
   if (!env->toc_init) _cdio_read_toc (env) ;
 
-  if (track_num == CDIO_CDROM_LEADOUT_TRACK) track_num = env->total_tracks+1;
+  if (i_track == CDIO_CDROM_LEADOUT_TRACK) i_track = env->total_tracks+1;
 
-  if (track_num > env->total_tracks+1 || track_num == 0)
+  if (i_track > env->total_tracks+1 || i_track == 0)
     return false;
 
-  switch (_cdio_get_track_format(env, track_num)) {
+  switch (_cdio_get_track_format(env, i_track)) {
+  case TRACK_FORMAT_XA:
+    return true;
   case TRACK_FORMAT_ERROR:
   case TRACK_FORMAT_CDI:
   case TRACK_FORMAT_AUDIO:
     return false;
+  case TRACK_FORMAT_DATA:
+    if (_AM_ASPI == env->access_mode ) 
+      return false;
   default:
     break;
   }
@@ -475,7 +530,7 @@ _cdio_get_track_green(void *obj, track_t track_num)
   /* FIXME: Dunno if this is the right way, but it's what 
      I was using in cd-info for a while.
    */
-  return ((env->tocent[track_num-1].Control & 2) != 0);
+  return ((env->tocent[i_track-1].Control & 2) != 0);
 }
 
 /*!  
@@ -579,8 +634,8 @@ bool
 cdio_is_device_win32(const char *source_name)
 {
   unsigned int len;
-  len = strlen(source_name);
 
+  len = strlen(source_name);
   if (NULL == source_name) return false;
 
 #ifdef HAVE_WIN32_CDROM
@@ -607,19 +662,40 @@ cdio_is_device_win32(const char *source_name)
   ones to set that up.
  */
 CdIo *
-cdio_open_win32 (const char *source_name)
+cdio_open_win32 (const char *psz_source_name)
+{
+#ifdef HAVE_WIN32_CDROM
+  if ( WIN_NT ) {
+    return cdio_open_am_win32(psz_source_name, "ioctl");
+  } else {
+    return cdio_open_am_win32(psz_source_name, "ASPI");
+  }
+#else 
+  return NULL;
+#endif /* HAVE_WIN32_CDROM */
+}
+
+/*!
+  Initialization routine. This is the only thing that doesn't
+  get called via a function pointer. In fact *we* are the
+  ones to set that up.
+ */
+CdIo *
+cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
 {
 
 #ifdef HAVE_WIN32_CDROM
   CdIo *ret;
   _img_private_t *_data;
+  char *psz_source;
 
   cdio_funcs _funcs = {
     .eject_media        = _cdio_eject_media,
-    .free               = _cdio_win32_free,
+    .free               = _free_win32,
     .get_arg            = _cdio_get_arg,
     .get_default_device = cdio_get_default_device_win32,
     .get_devices        = cdio_get_devices_win32,
+    .get_drive_cap      = _cdio_get_drive_cap,
     .get_first_track_num= _cdio_get_first_track_num,
     .get_mcn            = _cdio_get_mcn, 
     .get_num_tracks     = _cdio_get_num_tracks,
@@ -634,16 +710,31 @@ cdio_open_win32 (const char *source_name)
     .read_mode1_sectors = _cdio_read_mode1_sectors,
     .read_mode2_sector  = _cdio_read_mode2_sector,
     .read_mode2_sectors = _cdio_read_mode2_sectors,
-    .set_arg            = _cdio_set_arg,
+    .set_arg            = _set_arg_win32,
     .stat_size          = _cdio_stat_size
   };
 
   _data                 = _cdio_malloc (sizeof (_img_private_t));
+  _data->access_mode    = str_to_access_mode_win32(psz_access_mode);
   _data->gen.init       = false;
   _data->gen.fd         = -1;
 
-  _cdio_set_arg(_data, "source", (NULL == source_name) 
-		? cdio_get_default_device_win32(): source_name);
+  if (NULL == psz_orig_source) {
+    psz_source=cdio_get_default_device_win32();
+    if (NULL == psz_source) return NULL;
+    _set_arg_win32(_data, "source", psz_source);
+    free(psz_source);
+  } else {
+    if (cdio_is_device_win32(psz_orig_source))
+      _set_arg_win32(_data, "source", psz_orig_source);
+    else {
+      /* The below would be okay if all device drivers worked this way. */
+#if 0
+      cdio_info ("source %s is a not a device", psz_orig_source);
+#endif
+      return NULL;
+    }
+  }
 
   ret = cdio_new (_data, &_funcs);
   if (ret == NULL) return NULL;
@@ -651,7 +742,7 @@ cdio_open_win32 (const char *source_name)
   if (_cdio_init_win32(_data))
     return ret;
   else {
-    _cdio_win32_free (_data);
+    _free_win32 (_data);
     return NULL;
   }
   
