@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_linux.c,v 1.32 2004/03/07 02:42:22 rocky Exp $
+    $Id: _cdio_linux.c,v 1.56 2004/06/25 21:10:43 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -27,15 +27,15 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: _cdio_linux.c,v 1.32 2004/03/07 02:42:22 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_linux.c,v 1.56 2004/06/25 21:10:43 rocky Exp $";
 
 #include <string.h>
 
 #include <cdio/sector.h>
 #include <cdio/util.h>
+#include <cdio/scsi_mmc.h>
 #include "cdio_assert.h"
 #include "cdio_private.h"
-#include "scsi_mmc.h"
 
 #ifdef HAVE_LINUX_CDROM
 
@@ -67,20 +67,22 @@ static const char _rcsid[] = "$Id: _cdio_linux.c,v 1.32 2004/03/07 02:42:22 rock
 #include <sys/types.h>
 #include <sys/ioctl.h>
 
-#define TOTAL_TRACKS    (_obj->tochdr.cdth_trk1)
-#define FIRST_TRACK_NUM (_obj->tochdr.cdth_trk0)
+#define TOTAL_TRACKS    (env->tochdr.cdth_trk1)
+#define FIRST_TRACK_NUM (env->tochdr.cdth_trk0)
+
+typedef enum {
+  _AM_NONE,
+  _AM_IOCTL,
+  _AM_READ_CD,
+  _AM_READ_10
+} access_mode_t;
 
 typedef struct {
   /* Things common to all drivers like this. 
      This must be first. */
   generic_img_private_t gen; 
 
-  enum {
-    _AM_NONE,
-    _AM_IOCTL,
-    _AM_READ_CD,
-    _AM_READ_10
-  } access_mode;
+  access_mode_t access_mode;
 
   /* Track information */
   struct cdrom_tochdr    tochdr;
@@ -92,6 +94,25 @@ typedef struct {
 #define ERRNO_TRAYEMPTY(errno)	\
 	((errno == EIO) || (errno == ENOENT) || (errno == EINVAL))
 
+static access_mode_t 
+str_to_access_mode_linux(const char *psz_access_mode) 
+{
+  const access_mode_t default_access_mode = _AM_IOCTL;
+
+  if (NULL==psz_access_mode) return default_access_mode;
+  
+  if (!strcmp(psz_access_mode, "IOCTL"))
+    return _AM_IOCTL;
+  else if (!strcmp(psz_access_mode, "READ_CD"))
+    return _AM_READ_CD;
+  else if (!strcmp(psz_access_mode, "READ_10"))
+    return _AM_READ_10;
+  else {
+    cdio_warn ("unknown access type: %s. Default IOCTL used.", 
+	       psz_access_mode);
+    return default_access_mode;
+  }
+}
 
 /* Check a drive to see if it is a CD-ROM 
    Return 1 if a CD-ROM. 0 if it exists but isn't a CD-ROM drive
@@ -266,11 +287,11 @@ _cdio_mmc_read_sectors (int fd, void *buf, lba_t lba, int sector_type,
    Can read only up to 25 blocks.
 */
 static int
-_cdio_read_audio_sectors (void *env, void *buf, lsn_t lsn, 
-			  unsigned int nblocks)
+_read_audio_sectors_linux (void *user_data, void *buf, lsn_t lsn, 
+			   unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
-  return _cdio_mmc_read_sectors( _obj->gen.fd, buf, lsn, 
+  _img_private_t *env = user_data;
+  return _cdio_mmc_read_sectors( env->gen.fd, buf, lsn, 
 				 CDIO_MMC_READ_TYPE_CDDA, nblocks);
 }
 
@@ -278,26 +299,25 @@ _cdio_read_audio_sectors (void *env, void *buf, lsn_t lsn,
    Can read only up to 25 blocks.
 */
 static int
-__read_packet_mode2_sectors (int fd, void *buf, lba_t lba, 
-			     unsigned int nblocks, bool use_read_10)
+_read_packet_mode2_sectors_mmc (int fd, void *buf, lba_t lba, 
+				unsigned int nblocks, bool b_read_10)
 {
   struct cdrom_generic_command cgc;
 
   memset (&cgc, 0, sizeof (struct cdrom_generic_command));
 
-  cgc.cmd[0] = use_read_10 ? GPCMD_READ_10 : CDIO_MMC_GPCMD_READ_CD;
+  CDIO_MMC_SET_COMMAND(cgc.cmd, b_read_10 
+		       ? CDIO_MMC_GPCMD_READ_10 : CDIO_MMC_GPCMD_READ_CD);
 
   CDIO_MMC_SET_READ_LBA(cgc.cmd, lba);
   CDIO_MMC_SET_READ_LENGTH(cgc.cmd, nblocks);
 
-  if (!use_read_10)
-    {
-      cgc.cmd[1] = 0; /* sector size mode2 */
+  if (!b_read_10) {
+    cgc.cmd[1] = 0; /* sector size mode2 */
+    cgc.cmd[9] = 0x58; /* 2336 mode2 */
+  }
 
-      cgc.cmd[9] = 0x58; /* 2336 mode2 */
-    }
-
-  cgc.buflen = 2336 * nblocks;
+  cgc.buflen = M2RAW_SECTOR_SIZE * nblocks;
   cgc.buffer = buf;
 
 #ifdef HAVE_LINUX_CDROM_TIMEOUT
@@ -305,20 +325,20 @@ __read_packet_mode2_sectors (int fd, void *buf, lba_t lba,
 #endif
   cgc.data_direction = CGC_DATA_READ;
 
-  if (use_read_10)
+  if (b_read_10)
     {
       int retval;
 
-      if ((retval = _set_bsize (fd, 2336)))
+      if ((retval = _set_bsize (fd, M2RAW_SECTOR_SIZE)))
 	return retval;
 
       if ((retval = ioctl (fd, CDROM_SEND_PACKET, &cgc)))
 	{
-	  _set_bsize (fd, 2048);
+	  _set_bsize (fd, CDIO_CD_FRAMESIZE);
 	  return retval;
 	}
 
-      if ((retval = _set_bsize (fd, 2048)))
+      if ((retval = _set_bsize (fd, CDIO_CD_FRAMESIZE)))
 	return retval;
     }
   else
@@ -329,7 +349,7 @@ __read_packet_mode2_sectors (int fd, void *buf, lba_t lba,
 
 static int
 _read_packet_mode2_sectors (int fd, void *buf, lba_t lba, 
-			    unsigned int nblocks, bool use_read_10)
+			    unsigned int nblocks, bool b_read_10)
 {
   unsigned int l = 0;
   int retval = 0;
@@ -337,10 +357,10 @@ _read_packet_mode2_sectors (int fd, void *buf, lba_t lba,
   while (nblocks > 0)
     {
       const unsigned nblocks2 = (nblocks > 25) ? 25 : nblocks;
-      void *buf2 = ((char *)buf ) + (l * 2336);
+      void *buf2 = ((char *)buf ) + (l * M2RAW_SECTOR_SIZE);
       
-      retval |= __read_packet_mode2_sectors (fd, buf2, lba + l, nblocks2, 
-					     use_read_10);
+      retval |= _read_packet_mode2_sectors_mmc (fd, buf2, lba + l, nblocks2, 
+						b_read_10);
 
       if (retval)
 	break;
@@ -357,7 +377,7 @@ _read_packet_mode2_sectors (int fd, void *buf, lba_t lba,
    from lsn. Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sector (void *env, void *data, lsn_t lsn, 
+_read_mode1_sector_linux (void *env, void *data, lsn_t lsn, 
 			 bool b_form2)
 {
 
@@ -377,7 +397,7 @@ _cdio_read_mode1_sector (void *env, void *data, lsn_t lsn,
   switch (_obj->access_mode)
     {
     case _AM_NONE:
-      cdio_error ("no way to read mode1");
+      cdio_warn ("no way to read mode1");
       return 1;
       break;
       
@@ -432,16 +452,16 @@ _cdio_read_mode1_sector (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn, 
+_read_mode1_sectors_linux (void *user_data, void *data, lsn_t lsn, 
 			  bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   unsigned int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _cdio_read_mode1_sector (_obj, 
+    if ( (retval = _read_mode1_sector_linux (env, 
 					    ((char *)data) + (blocksize * i),
 					    lsn + i, b_form2)) )
       return retval;
@@ -454,14 +474,14 @@ _cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn,
    from lsn. Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sector (void *env, void *data, lsn_t lsn, 
-			 bool b_form2)
+_read_mode2_sector_linux (void *user_data, void *data, lsn_t lsn, 
+			  bool b_form2)
 {
   char buf[M2RAW_SECTOR_SIZE] = { 0, };
   struct cdrom_msf *msf = (struct cdrom_msf *) &buf;
   msf_t _msf;
 
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
   msf->cdmsf_min0 = from_bcd8(_msf.m);
@@ -469,15 +489,15 @@ _cdio_read_mode2_sector (void *env, void *data, lsn_t lsn,
   msf->cdmsf_frame0 = from_bcd8(_msf.f);
 
  retry:
-  switch (_obj->access_mode)
+  switch (env->access_mode)
     {
     case _AM_NONE:
-      cdio_error ("no way to read mode2");
+      cdio_warn ("no way to read mode2");
       return 1;
       break;
       
     case _AM_IOCTL:
-      if (ioctl (_obj->gen.fd, CDROMREADMODE2, &buf) == -1)
+      if (ioctl (env->gen.fd, CDROMREADMODE2, &buf) == -1)
 	{
 	  perror ("ioctl()");
 	  return 1;
@@ -487,20 +507,20 @@ _cdio_read_mode2_sector (void *env, void *data, lsn_t lsn,
       
     case _AM_READ_CD:
     case _AM_READ_10:
-      if (_read_packet_mode2_sectors (_obj->gen.fd, buf, lsn, 1, 
-				      (_obj->access_mode == _AM_READ_10)))
+      if (_read_packet_mode2_sectors (env->gen.fd, buf, lsn, 1, 
+				      (env->access_mode == _AM_READ_10)))
 	{
 	  perror ("ioctl()");
-	  if (_obj->access_mode == _AM_READ_CD)
+	  if (env->access_mode == _AM_READ_CD)
 	    {
 	      cdio_info ("READ_CD failed; switching to READ_10 mode...");
-	      _obj->access_mode = _AM_READ_10;
+	      env->access_mode = _AM_READ_10;
 	      goto retry;
 	    }
 	  else
 	    {
 	      cdio_info ("READ_10 failed; switching to ioctl(CDROMREADMODE2) mode...");
-	      _obj->access_mode = _AM_IOCTL;
+	      env->access_mode = _AM_IOCTL;
 	      goto retry;
 	    }
 	  return 1;
@@ -522,17 +542,18 @@ _cdio_read_mode2_sector (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn, 
+_read_mode2_sectors_linux (void *user_data, void *data, lsn_t lsn, 
 			  bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   unsigned int i;
-  int retval;
-  unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
+  unsigned int i_blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
+  /* For each frame, pick out the data part we need */
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _cdio_read_mode2_sector (_obj, 
-					    ((char *)data) + (blocksize * i),
+    int retval;
+    if ( (retval = _read_mode2_sector_linux (env, 
+					    ((char *)data) + (i_blocksize * i),
 					    lsn + i, b_form2)) )
       return retval;
   }
@@ -543,16 +564,16 @@ _cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn,
    Return the size of the CD in logical block address (LBA) units.
  */
 static uint32_t 
-_cdio_stat_size (void *env)
+_stat_size_linux (void *user_data)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   struct cdrom_tocentry tocent;
   uint32_t size;
 
   tocent.cdte_track = CDIO_CDROM_LEADOUT_TRACK;
   tocent.cdte_format = CDROM_LBA;
-  if (ioctl (_obj->gen.fd, CDROMREADTOCENTRY, &tocent) == -1)
+  if (ioctl (env->gen.fd, CDROMREADTOCENTRY, &tocent) == -1)
     {
       perror ("ioctl(CDROMREADTOCENTRY)");
       exit (EXIT_FAILURE);
@@ -572,30 +593,22 @@ _cdio_stat_size (void *env)
   0 is returned if no error was found, and nonzero if there as an error.
 */
 static int
-_cdio_set_arg (void *env, const char key[], const char value[])
+_set_arg_linux (void *user_data, const char key[], const char value[])
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   if (!strcmp (key, "source"))
     {
       if (!value)
 	return -2;
 
-      free (_obj->gen.source_name);
+      free (env->gen.source_name);
       
-      _obj->gen.source_name = strdup (value);
+      env->gen.source_name = strdup (value);
     }
   else if (!strcmp (key, "access-mode"))
     {
-      if (!strcmp(value, "IOCTL"))
-	_obj->access_mode = _AM_IOCTL;
-      else if (!strcmp(value, "READ_CD"))
-	_obj->access_mode = _AM_READ_CD;
-      else if (!strcmp(value, "READ_10"))
-	_obj->access_mode = _AM_READ_10;
-      else {
-	cdio_warn ("unknown access type: %s. ignored.", value);
-      }
+      return str_to_access_mode_linux(value);
     }
   else 
     return -1;
@@ -608,29 +621,30 @@ _cdio_set_arg (void *env, const char key[], const char value[])
   Return false if successful or true if an error.
 */
 static bool
-_cdio_read_toc (_img_private_t *_obj) 
+_cdio_read_toc (_img_private_t *env) 
 {
   int i;
 
   /* read TOC header */
-  if ( ioctl(_obj->gen.fd, CDROMREADTOCHDR, &_obj->tochdr) == -1 ) {
-    cdio_error("%s: %s\n", 
+  if ( ioctl(env->gen.fd, CDROMREADTOCHDR, &env->tochdr) == -1 ) {
+    cdio_warn("%s: %s\n", 
             "error in ioctl CDROMREADTOCHDR", strerror(errno));
     return false;
   }
 
   /* read individual tracks */
   for (i= FIRST_TRACK_NUM; i<=TOTAL_TRACKS; i++) {
-    _obj->tocent[i-1].cdte_track = i;
-    _obj->tocent[i-1].cdte_format = CDROM_MSF;
-    if ( ioctl(_obj->gen.fd, CDROMREADTOCENTRY, &_obj->tocent[i-1]) == -1 ) {
-      cdio_error("%s %d: %s\n",
+    env->tocent[i-FIRST_TRACK_NUM].cdte_track = i;
+    env->tocent[i-FIRST_TRACK_NUM].cdte_format = CDROM_MSF;
+    if ( ioctl(env->gen.fd, CDROMREADTOCENTRY, 
+	       &env->tocent[i-FIRST_TRACK_NUM]) == -1 ) {
+      cdio_warn("%s %d: %s\n",
               "error in ioctl CDROMREADTOCENTRY for track", 
               i, strerror(errno));
       return false;
     }
     /****
-    struct cdrom_msf0 *msf= &_obj->tocent[i-1].cdte_addr.msf;
+    struct cdrom_msf0 *msf= &env->tocent[i-1].cdte_addr.msf;
     
     fprintf (stdout, "--- track# %d (msf %2.2x:%2.2x:%2.2x)\n",
 	     i, msf->minute, msf->second, msf->frame);
@@ -639,25 +653,25 @@ _cdio_read_toc (_img_private_t *_obj)
   }
 
   /* read the lead-out track */
-  _obj->tocent[TOTAL_TRACKS].cdte_track = CDIO_CDROM_LEADOUT_TRACK;
-  _obj->tocent[TOTAL_TRACKS].cdte_format = CDROM_MSF;
+  env->tocent[TOTAL_TRACKS].cdte_track = CDIO_CDROM_LEADOUT_TRACK;
+  env->tocent[TOTAL_TRACKS].cdte_format = CDROM_MSF;
 
-  if (ioctl(_obj->gen.fd, CDROMREADTOCENTRY, 
-	    &_obj->tocent[TOTAL_TRACKS]) == -1 ) {
-    cdio_error("%s: %s\n", 
+  if (ioctl(env->gen.fd, CDROMREADTOCENTRY, 
+	    &env->tocent[TOTAL_TRACKS]) == -1 ) {
+    cdio_warn("%s: %s\n", 
 	     "error in ioctl CDROMREADTOCENTRY for lead-out",
             strerror(errno));
     return false;
   }
 
   /*
-  struct cdrom_msf0 *msf= &_obj->tocent[TOTAL_TRACKS].cdte_addr.msf;
+  struct cdrom_msf0 *msf= &env->tocent[TOTAL_TRACKS].cdte_addr.msf;
 
   fprintf (stdout, "--- track# %d (msf %2.2x:%2.2x:%2.2x)\n",
 	   i, msf->minute, msf->second, msf->frame);
   */
 
-  _obj->gen.toc_init = true;
+  env->gen.toc_init = true;
   return true;
 }
 
@@ -665,7 +679,7 @@ _cdio_read_toc (_img_private_t *_obj)
  * Eject using SCSI commands. Return 1 if successful, 0 otherwise.
  */
 static int 
-_cdio_eject_scsi(int fd)
+_eject_media_mmc(int fd)
 {
   int status;
   struct sdata {
@@ -688,7 +702,7 @@ _cdio_eject_scsi(int fd)
   
   scsi_cmd.inlen  = 0;
   scsi_cmd.outlen = 0;
-  scsi_cmd.cmd[0] = START_STOP;
+  CDIO_MMC_SET_COMMAND(scsi_cmd.cmd, CDIO_MMC_START_STOP);
   scsi_cmd.cmd[1] = 0;
   scsi_cmd.cmd[2] = 0;
   scsi_cmd.cmd[3] = 0;
@@ -700,7 +714,7 @@ _cdio_eject_scsi(int fd)
   
   scsi_cmd.inlen  = 0;
   scsi_cmd.outlen = 0;
-  scsi_cmd.cmd[0] = START_STOP;
+  CDIO_MMC_SET_COMMAND(scsi_cmd.cmd, CDIO_MMC_START_STOP);
   scsi_cmd.cmd[1] = 0;
   scsi_cmd.cmd[2] = 0;
   scsi_cmd.cmd[3] = 0;
@@ -720,41 +734,41 @@ _cdio_eject_scsi(int fd)
   Return 0 if success and 1 for failure, and 2 if no routine.
  */
 static int 
-_cdio_eject_media (void *env) {
+_eject_media_linux (void *user_data) {
 
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   int ret=2;
   int status;
   int fd;
 
-  close(_obj->gen.fd);
-  _obj->gen.fd = -1;
-  if ((fd = open (_obj->gen.source_name, O_RDONLY|O_NONBLOCK)) > -1) {
+  close(env->gen.fd);
+  env->gen.fd = -1;
+  if ((fd = open (env->gen.source_name, O_RDONLY|O_NONBLOCK)) > -1) {
     if((status = ioctl(fd, CDROM_DRIVE_STATUS, CDSL_CURRENT)) > 0) {
       switch(status) {
       case CDS_TRAY_OPEN:
 	if((ret = ioctl(fd, CDROMCLOSETRAY)) != 0) {
-	  cdio_error ("ioctl CDROMCLOSETRAY failed: %s\n", strerror(errno));  
+	  cdio_warn ("ioctl CDROMCLOSETRAY failed: %s\n", strerror(errno));  
 	  ret = 1;
 	}
 	break;
       case CDS_DISC_OK:
 	if((ret = ioctl(fd, CDROMEJECT)) != 0) {
 	  int eject_error = errno;
-	  /* Try ejecting the SCSI way... */
-	  ret = _cdio_eject_scsi(fd);
+	  /* Try ejecting the MMC way... */
+	  ret = _eject_media_mmc(fd);
 	  if (0 != ret) {
-	    cdio_error("ioctl CDROMEJECT failed: %s\n", strerror(eject_error));
+	    cdio_warn("ioctl CDROMEJECT failed: %s\n", strerror(eject_error));
 	    ret = 1;
 	  }
 	}
 	break;
       default:
-	cdio_error ("Unknown CD-ROM (%d)\n", status);
+	cdio_warn ("Unknown CD-ROM (%d)\n", status);
 	ret = 1;
       }
     } else {
-      cdio_error ("CDROM_DRIVE_STATUS failed: %s\n", strerror(errno));
+      cdio_warn ("CDROM_DRIVE_STATUS failed: %s\n", strerror(errno));
       ret=1;
     }
     close(fd);
@@ -767,7 +781,7 @@ _cdio_eject_media (void *env) {
   Return the value associated with the key "arg".
 */
 static const char *
-_cdio_get_arg (void *env, const char key[])
+_get_arg_linux (void *env, const char key[])
 {
   _img_private_t *_obj = env;
 
@@ -793,11 +807,11 @@ _cdio_get_arg (void *env, const char key[])
   CDIO_INVALID_TRACK is returned on error.
 */
 static track_t
-_cdio_get_first_track_num(void *env) 
+_get_first_track_num_linux(void *user_data) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
+  if (!env->gen.toc_init) _cdio_read_toc (env) ;
 
   return FIRST_TRACK_NUM;
 }
@@ -810,26 +824,46 @@ _cdio_get_first_track_num(void *env)
 
  */
 static char *
-_cdio_get_mcn (void *env) {
+_get_mcn_linux (const void *env) {
 
   struct cdrom_mcn mcn;
-  _img_private_t *_obj = env;
+  const _img_private_t *_obj = env;
   memset(&mcn, 0, sizeof(mcn));
   if (ioctl(_obj->gen.fd, CDROM_GET_MCN, &mcn) != 0)
     return NULL;
   return strdup(mcn.medium_catalog_number);
 }
 
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+static cdio_drive_cap_t
+_get_drive_cap_linux (const void *env) {
+  const _img_private_t *_obj = env;
+  int32_t i_drivetype;
+
+  i_drivetype = ioctl (_obj->gen.fd, CDROM_GET_CAPABILITY, CDSL_CURRENT);
+
+  if (i_drivetype < 0) return CDIO_DRIVE_CAP_ERROR;
+
+  /* If >= 0 we can safely cast as cdio_drive_cap_t and return */
+  return (cdio_drive_cap_t) i_drivetype;
+}
+
 /*!
   Return the number of tracks in the current medium.
   CDIO_INVALID_TRACK is returned on error.
 */
 static track_t
-_cdio_get_num_tracks(void *env) 
+_get_num_tracks_linux(void *user_data) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
+  if (!env->gen.toc_init) _cdio_read_toc (env) ;
 
   return TOTAL_TRACKS;
 }
@@ -838,22 +872,22 @@ _cdio_get_num_tracks(void *env)
   Get format of track. 
 */
 static track_format_t
-_cdio_get_track_format(void *env, track_t track_num) 
+_get_track_format_linux(void *user_data, track_t i_track) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
-
-  if (track_num > TOTAL_TRACKS || track_num == 0)
+  if (i_track > (TOTAL_TRACKS+FIRST_TRACK_NUM) || i_track < FIRST_TRACK_NUM)
     return TRACK_FORMAT_ERROR;
 
+  i_track -= FIRST_TRACK_NUM;
+
   /* This is pretty much copied from the "badly broken" cdrom_count_tracks
      in linux/cdrom.c.
    */
-  if (_obj->tocent[track_num-1].cdte_ctrl & CDROM_DATA_TRACK) {
-    if (_obj->tocent[track_num-1].cdte_format == 0x10)
+  if (env->tocent[i_track].cdte_ctrl & CDIO_CDROM_DATA_TRACK) {
+    if (env->tocent[i_track].cdte_format == CDIO_CDROM_CDI_TRACK)
       return TRACK_FORMAT_CDI;
-    else if (_obj->tocent[track_num-1].cdte_format == 0x20) 
+    else if (env->tocent[i_track].cdte_format == CDIO_CDROM_XA_TRACK)
       return TRACK_FORMAT_XA;
     else
       return TRACK_FORMAT_DATA;
@@ -871,45 +905,48 @@ _cdio_get_track_format(void *env, track_t track_num)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *env, track_t track_num) 
+_get_track_green_linux(void *user_data, track_t i_track) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
+  if (!env->gen.toc_init) _cdio_read_toc (env) ;
 
-  if (track_num == CDIO_CDROM_LEADOUT_TRACK) track_num = TOTAL_TRACKS+1;
-
-  if (track_num > TOTAL_TRACKS+1 || track_num == 0)
+  if (i_track > (TOTAL_TRACKS+FIRST_TRACK_NUM) || i_track < FIRST_TRACK_NUM)
     return false;
 
+  i_track -= FIRST_TRACK_NUM;
+
   /* FIXME: Dunno if this is the right way, but it's what 
-     I was using in cdinfo for a while.
+     I was using in cd-info for a while.
    */
-  return ((_obj->tocent[track_num-1].cdte_ctrl & 2) != 0);
+  return ((env->tocent[i_track].cdte_ctrl & 2) != 0);
 }
 
 /*!  
   Return the starting MSF (minutes/secs/frames) for track number
-  track_num in obj.  Track numbers start at 1.
+  track_num in obj.  Track numbers usually start at something 
+  greater than 0, usually 1.
+
   The "leadout" track is specified either by
-  using track_num LEADOUT_TRACK or the total tracks+1.
+  using i_track LEADOUT_TRACK or the total tracks+1.
   False is returned if there is no track entry.
 */
 static bool
-_cdio_get_track_msf(void *env, track_t track_num, msf_t *msf)
+_get_track_msf_linux(void *user_data, track_t i_track, msf_t *msf)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   if (NULL == msf) return false;
 
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
+  if (!env->gen.toc_init) _cdio_read_toc (env) ;
 
-  if (track_num == CDIO_CDROM_LEADOUT_TRACK) track_num = TOTAL_TRACKS+1;
+  if (i_track == CDIO_CDROM_LEADOUT_TRACK) 
+    i_track = TOTAL_TRACKS + FIRST_TRACK_NUM;
 
-  if (track_num > TOTAL_TRACKS+1 || track_num == 0) {
+  if (i_track > (TOTAL_TRACKS+FIRST_TRACK_NUM) || i_track < FIRST_TRACK_NUM) {
     return false;
   } else {
-    struct cdrom_msf0  *msf0= &_obj->tocent[track_num-1].cdte_addr.msf;
+    struct cdrom_msf0  *msf0= &env->tocent[i_track-FIRST_TRACK_NUM].cdte_addr.msf;
     msf->m = to_bcd8(msf0->minute);
     msf->s = to_bcd8(msf0->second);
     msf->f = to_bcd8(msf0->frame);
@@ -1046,50 +1083,72 @@ cdio_get_default_device_linux(void)
   ones to set that up.
  */
 CdIo *
-cdio_open_linux (const char *orig_source_name)
+cdio_open_linux (const char *psz_source_name)
+{
+  return cdio_open_am_linux(psz_source_name, NULL);
+}
+
+/*!
+  Initialization routine. This is the only thing that doesn't
+  get called via a function pointer. In fact *we* are the
+  ones to set that up.
+ */
+CdIo *
+cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
 {
 
 #ifdef HAVE_LINUX_CDROM
   CdIo *ret;
   _img_private_t *_data;
-  char *source_name;
+  char *psz_source;
 
   cdio_funcs _funcs = {
-    .eject_media        = _cdio_eject_media,
+    .eject_media        = _eject_media_linux,
     .free               = cdio_generic_free,
-    .get_arg            = _cdio_get_arg,
+    .get_arg            = _get_arg_linux,
     .get_devices        = cdio_get_devices_linux,
     .get_default_device = cdio_get_default_device_linux,
-    .get_first_track_num= _cdio_get_first_track_num,
-    .get_mcn            = _cdio_get_mcn,
-    .get_num_tracks     = _cdio_get_num_tracks,
-    .get_track_format   = _cdio_get_track_format,
-    .get_track_green    = _cdio_get_track_green,
+    .get_drive_cap      = _get_drive_cap_linux,
+    .get_first_track_num= _get_first_track_num_linux,
+    .get_mcn            = _get_mcn_linux,
+    .get_num_tracks     = _get_num_tracks_linux,
+    .get_track_format   = _get_track_format_linux,
+    .get_track_green    = _get_track_green_linux,
     .get_track_lba      = NULL, /* This could be implemented if need be. */
-    .get_track_msf      = _cdio_get_track_msf,
+    .get_track_msf      = _get_track_msf_linux,
     .lseek              = cdio_generic_lseek,
     .read               = cdio_generic_read,
-    .read_audio_sectors = _cdio_read_audio_sectors,
-    .read_mode1_sector  = _cdio_read_mode1_sector,
-    .read_mode1_sectors = _cdio_read_mode1_sectors,
-    .read_mode2_sector  = _cdio_read_mode2_sector,
-    .read_mode2_sectors = _cdio_read_mode2_sectors,
-    .set_arg            = _cdio_set_arg,
-    .stat_size          = _cdio_stat_size
+    .read_audio_sectors = _read_audio_sectors_linux,
+    .read_mode1_sector  = _read_mode1_sector_linux,
+    .read_mode1_sectors = _read_mode1_sectors_linux,
+    .read_mode2_sector  = _read_mode2_sector_linux,
+    .read_mode2_sectors = _read_mode2_sectors_linux,
+    .set_arg            = _set_arg_linux,
+    .stat_size          = _stat_size_linux
   };
 
   _data                 = _cdio_malloc (sizeof (_img_private_t));
-  _data->access_mode    = _AM_READ_CD;
+
+  _data->access_mode    = str_to_access_mode_linux(access_mode);
   _data->gen.init       = false;
   _data->gen.fd         = -1;
 
-  if (NULL == orig_source_name) {
-    source_name=cdio_get_default_device_linux();
-    if (NULL == source_name) return NULL;
-    _cdio_set_arg(_data, "source", source_name);
-    free(source_name);
-  } else 
-    _cdio_set_arg(_data, "source", orig_source_name);
+  if (NULL == psz_orig_source) {
+    psz_source=cdio_get_default_device_linux();
+    if (NULL == psz_source) return NULL;
+    _set_arg_linux(_data, "source", psz_source);
+    free(psz_source);
+  } else {
+    if (cdio_is_device_generic(psz_orig_source))
+      _set_arg_linux(_data, "source", psz_orig_source);
+    else {
+      /* The below would be okay if all device drivers worked this way. */
+#if 0
+      cdio_info ("source %s is a not a device", psz_orig_source);
+#endif
+      return NULL;
+    }
+  }
 
   ret = cdio_new (_data, &_funcs);
   if (ret == NULL) return NULL;
