@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_osx.c,v 1.15 2004/03/20 04:16:59 rocky Exp $
+    $Id: _cdio_osx.c,v 1.46 2004/06/24 07:51:51 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com> 
     from vcdimager code: 
@@ -9,6 +9,7 @@
                Gildas Bazin <gbazin@netcourrier.com>
                Jon Lech Johansen <jon-vl@nanocrew.net>
                Derk-Jan Hartman <hartman at videolan.org>
+               Justin F. Hallett <thesin@southofheaven.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,10 +31,10 @@
 */
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: _cdio_osx.c,v 1.15 2004/03/20 04:16:59 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_osx.c,v 1.46 2004/06/24 07:51:51 rocky Exp $";
 
 #include <cdio/sector.h>
 #include <cdio/util.h>
@@ -61,65 +62,50 @@ static const char _rcsid[] = "$Id: _cdio_osx.c,v 1.15 2004/03/20 04:16:59 rocky
 #include <IOKit/IOKitLib.h>
 #include <IOKit/IOBSD.h>
 #include <IOKit/storage/IOCDTypes.h>
+#include <IOKit/storage/IODVDTypes.h>
 #include <IOKit/storage/IOMedia.h>
 #include <IOKit/storage/IOCDMedia.h>
 #include <IOKit/storage/IOCDMediaBSDClient.h>
+#include <IOKit/storage/IODVDMediaBSDClient.h>
 
-#define TOTAL_TRACKS    (_obj->num_tracks)
+/* Note leadout is normally defined 0xAA, But on OSX 0xA0 is "lead in" while
+   0xA2 is "lead out". Don't ask me why. */
+#define	OSX_CDROM_LEADOUT_TRACK 0xA2
+
+#define TOTAL_TRACKS    (env->i_last_track - env->i_first_track + 1)
+
+#define CDROM_CDI_TRACK 0x1
+#define CDROM_XA_TRACK  0x2
+
+typedef enum {
+  _AM_NONE,
+  _AM_OSX,
+} access_mode_t;
 
 typedef struct {
   /* Things common to all drivers like this. 
      This must be first. */
   generic_img_private_t gen; 
 
-  enum {
-    _AM_NONE,
-    _AM_OSX,
-  } access_mode;
+  access_mode_t access_mode;
 
   /* Track information */
-  bool toc_init;                         /* if true, info below is valid. */
+  bool toc_init;             /* if true, info below is valid. */
   CDTOC *pTOC;
   int i_descriptors;
-  track_t num_tracks;
+  track_t i_last_track;      /* highest track number */
+  track_t i_first_track;     /* first track */
   lsn_t   *pp_lba;
 
 } _img_private_t;
 
 static void 
-_cdio_osx_free (void *env) {
-  _img_private_t *_obj = env;
-  if (NULL == _obj) return;
-  cdio_generic_free(_obj);
-  if (NULL != _obj->pp_lba) free((void *) _obj->pp_lba);
-  if (NULL != _obj->pTOC) free((void *) _obj->pTOC);
-}
-
-/****************************************************************************
-  cdio_getNumberOfTracks: get number of tracks in TOC 
-  This is an internal routine and is called once per CD open.
- ****************************************************************************/
-static track_t
-_cdio_getNumberOfTracks( CDTOC *pTOC, int i_descriptors )
-{
-    track_t track = CDIO_INVALID_TRACK; 
-    int i;
-    int i_tracks = 0;
-    CDTOCDescriptor *pTrackDescriptors;
-
-    pTrackDescriptors = pTOC->descriptors;
-
-    for( i = i_descriptors; i >= 0; i-- )
-    {
-        track = pTrackDescriptors[i].point;
-
-	if( track > CDIO_CD_MAX_TRACKS || track < CDIO_CD_MIN_TRACK_NO )
-            continue;
-
-        i_tracks++; 
-    }
-
-    return( i_tracks );
+_free_osx (void *user_data) {
+  _img_private_t *env = user_data;
+  if (NULL == env) return;
+  cdio_generic_free(env);
+  if (NULL != env->pp_lba) free((void *) env->pp_lba);
+  if (NULL != env->pTOC) free((void *) env->pTOC);
 }
 
 /*!
@@ -128,10 +114,10 @@ _cdio_getNumberOfTracks( CDTOC *pTOC, int i_descriptors )
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn, 
-			  bool is_form2, unsigned int nblocks)
+_get_read_mode1_sectors_osx (void *user_data, void *data, lsn_t lsn, 
+			     bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   dk_cd_read_t cd_read;
   
   memset( &cd_read, 0, sizeof(cd_read) );
@@ -140,7 +126,7 @@ _cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn,
   cd_read.buffer      = data;
   cd_read.sectorType  = kCDSectorTypeMode1;
   
-  if (is_form2) {
+  if (b_form2) {
     cd_read.offset       = lsn * kCDSectorSizeMode2;
     cd_read.bufferLength = kCDSectorSizeMode2 * nblocks;
   } else {
@@ -148,9 +134,9 @@ _cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn,
     cd_read.bufferLength = kCDSectorSizeMode1 * nblocks;
   }
   
-   if( ioctl( _obj->gen.fd, DKIOCCDREAD, &cd_read ) == -1 )
+   if( ioctl( env->gen.fd, DKIOCCDREAD, &cd_read ) == -1 )
   {
-    cdio_error( "could not read block %d, %s", lsn, strerror(errno) );
+    cdio_info( "could not read block %d, %s", lsn, strerror(errno) );
     return -1;
   }
   return 0;
@@ -163,10 +149,10 @@ _cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn, 
-			  bool is_form2, unsigned int nblocks)
+_get_read_mode2_sectors_osx (void *user_data, void *data, lsn_t lsn, 
+			     bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   dk_cd_read_t cd_read;
   
   memset( &cd_read, 0, sizeof(cd_read) );
@@ -174,7 +160,7 @@ _cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn,
   cd_read.sectorArea = kCDSectorAreaUser;
   cd_read.buffer = data;
   
-  if (is_form2) {
+  if (b_form2) {
     cd_read.offset       = lsn * kCDSectorSizeMode2Form2;
     cd_read.sectorType   = kCDSectorTypeMode2Form2;
     cd_read.bufferLength = kCDSectorSizeMode2Form2 * nblocks;
@@ -184,9 +170,9 @@ _cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn,
     cd_read.bufferLength = kCDSectorSizeMode2Form1 * nblocks;
   }
   
-  if( ioctl( _obj->gen.fd, DKIOCCDREAD, &cd_read ) == -1 )
+  if( ioctl( env->gen.fd, DKIOCCDREAD, &cd_read ) == -1 )
   {
-    cdio_error( "could not read block %d, %s", lsn, strerror(errno) );
+    cdio_info( "could not read block %d, %s", lsn, strerror(errno) );
     return -1;
   }
   return 0;
@@ -198,10 +184,10 @@ _cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_audio_sectors (void *env, void *data, lsn_t lsn, 
-			  unsigned int nblocks)
+_get_read_audio_sectors_osx (void *user_data, void *data, lsn_t lsn, 
+			     unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   dk_cd_read_t cd_read;
   
   memset( &cd_read, 0, sizeof(cd_read) );
@@ -213,9 +199,9 @@ _cdio_read_audio_sectors (void *env, void *data, lsn_t lsn,
   cd_read.buffer = data;
   cd_read.bufferLength = kCDSectorSizeCDDA * nblocks;
   
-  if( ioctl( _obj->gen.fd, DKIOCCDREAD, &cd_read ) == -1 )
+  if( ioctl( env->gen.fd, DKIOCCDREAD, &cd_read ) == -1 )
   {
-    cdio_error( "could not read block %d", lsn );
+    cdio_info( "could not read block %d", lsn );
     return -1;
   }
   return 0;
@@ -226,10 +212,10 @@ _cdio_read_audio_sectors (void *env, void *data, lsn_t lsn,
    from lsn. Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sector (void *env, void *data, lsn_t lsn, 
-			 bool is_form2)
+_get_read_mode1_sector_osx (void *user_data, void *data, lsn_t lsn, 
+			    bool b_form2)
 {
-  return _cdio_read_mode1_sectors(env, data, lsn, is_form2, 1);
+  return _get_read_mode1_sectors_osx(user_data, data, lsn, b_form2, 1);
 }
 
 /*!
@@ -237,35 +223,35 @@ _cdio_read_mode1_sector (void *env, void *data, lsn_t lsn,
    from lsn. Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sector (void *env, void *data, lsn_t lsn, 
-			 bool is_form2)
+_get_read_mode2_sector_osx (void *user_data, void *data, lsn_t lsn, 
+			    bool b_form2)
 {
-  return _cdio_read_mode2_sectors(env, data, lsn, is_form2, 1);
+  return _get_read_mode2_sectors_osx(user_data, data, lsn, b_form2, 1);
 }
 
 /*!
   Set the key "arg" to "value" in source device.
 */
 static int
-_cdio_set_arg (void *env, const char key[], const char value[])
+_set_arg_osx (void *user_data, const char key[], const char value[])
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   if (!strcmp (key, "source"))
     {
       if (!value)
 	return -2;
 
-      free (_obj->gen.source_name);
+      free (env->gen.source_name);
       
-      _obj->gen.source_name = strdup (value);
+      env->gen.source_name = strdup (value);
     }
   else if (!strcmp (key, "access-mode"))
     {
       if (!strcmp(value, "OSX"))
-	_obj->access_mode = _AM_OSX;
+	env->access_mode = _AM_OSX;
       else
-	cdio_error ("unknown access type: %s. ignored.", value);
+	cdio_warn ("unknown access type: %s. ignored.", value);
     }
   else 
     return -1;
@@ -278,7 +264,7 @@ _cdio_set_arg (void *env, const char key[], const char value[])
   Return false if successful or true if an error.
 */
 static bool
-_cdio_read_toc (_img_private_t *_obj) 
+_cdio_read_toc (_img_private_t *env) 
 {
   mach_port_t port;
   char *psz_devname;
@@ -288,18 +274,18 @@ _cdio_read_toc (_img_private_t *_obj)
   CFMutableDictionaryRef properties;
   CFDataRef data;
   
-  _obj->gen.fd = open( _obj->gen.source_name, O_RDONLY | O_NONBLOCK );
-  if (-1 == _obj->gen.fd) {
-    cdio_error("Failed to open %s: %s", _obj->gen.source_name,
+  env->gen.fd = open( env->gen.source_name, O_RDONLY | O_NONBLOCK );
+  if (-1 == env->gen.fd) {
+    cdio_warn("Failed to open %s: %s", env->gen.source_name,
 	       strerror(errno));
     return false;
   }
 
   /* get the device name */
-  if( ( psz_devname = strrchr( _obj->gen.source_name, '/') ) != NULL )
+  if( ( psz_devname = strrchr( env->gen.source_name, '/') ) != NULL )
     ++psz_devname;
   else
-    psz_devname = _obj->gen.source_name;
+    psz_devname = env->gen.source_name;
   
   /* unraw the device name */
   if( *psz_devname == 'r' )
@@ -308,7 +294,7 @@ _cdio_read_toc (_img_private_t *_obj)
   /* get port for IOKit communication */
   if( ( ret = IOMasterPort( MACH_PORT_NULL, &port ) ) != KERN_SUCCESS )
     {
-      cdio_error( "IOMasterPort: 0x%08x", ret );
+      cdio_warn( "IOMasterPort: 0x%08x", ret );
       return false;
     }
   
@@ -317,7 +303,7 @@ _cdio_read_toc (_img_private_t *_obj)
 					   port, IOBSDNameMatching( port, 0, psz_devname ),
 					   &iterator ) ) != KERN_SUCCESS )
     {
-        cdio_error( "IOServiceGetMatchingServices: 0x%08x", ret );
+        cdio_warn( "IOServiceGetMatchingServices: 0x%08x", ret );
         return false;
     }
   
@@ -333,7 +319,7 @@ _cdio_read_toc (_img_private_t *_obj)
 					      &iterator );
       if( ret != KERN_SUCCESS )
         {
-	  cdio_error( "IORegistryEntryGetParentIterator: 0x%08x", ret );
+	  cdio_warn( "IORegistryEntryGetParentIterator: 0x%08x", ret );
 	  IOObjectRelease( service );
 	  return false;
         }
@@ -345,7 +331,7 @@ _cdio_read_toc (_img_private_t *_obj)
   
   if( service == 0 )
     {
-      cdio_error( "search for kIOCDMediaClass came up empty" );
+      cdio_warn( "search for kIOCDMediaClass came up empty" );
       return false;
     }
   
@@ -355,7 +341,7 @@ _cdio_read_toc (_img_private_t *_obj)
   
   if(  ret != KERN_SUCCESS )
     {
-      cdio_error( "IORegistryEntryCreateCFProperties: 0x%08x", ret );
+      cdio_warn( "IORegistryEntryCreateCFProperties: 0x%08x", ret );
       IOObjectRelease( service );
       return false;
     }
@@ -371,70 +357,116 @@ _cdio_read_toc (_img_private_t *_obj)
       buf_len = CFDataGetLength( data ) + 1;
       range = CFRangeMake( 0, buf_len );
       
-      if( ( _obj->pTOC = (CDTOC *)malloc( buf_len ) ) != NULL ) {
-	CFDataGetBytes( data, range, (u_char *) _obj->pTOC );
+      if( ( env->pTOC = (CDTOC *)malloc( buf_len ) ) != NULL ) {
+	CFDataGetBytes( data, range, (u_char *) env->pTOC );
       } else {
-	cdio_error( "Trouble allocating CDROM TOC" );
+	cdio_warn( "Trouble allocating CDROM TOC" );
 	return false;
       }
     }
   else
     {
-      cdio_error( "CFDictionaryGetValue failed" );
+      cdio_warn( "CFDictionaryGetValue failed" );
+      return false;
     }
   
   CFRelease( properties );
   IOObjectRelease( service ); 
 
-  _obj->i_descriptors = CDTOCGetDescriptorCount ( _obj->pTOC );
-  _obj->num_tracks = _cdio_getNumberOfTracks(_obj->pTOC, _obj->i_descriptors);
+  env->i_descriptors = CDTOCGetDescriptorCount ( env->pTOC );
 
-  /* Read in starting sectors */
+  /* Read in starting sectors. There may be non-tracks mixed in with
+     the real tracks.  So find the first and last track number by
+     scanning. Also find the lead-out track position.
+   */
   {
     int i, i_leadout = -1;
-    CDTOCDescriptor *pTrackDescriptors;
-    track_t track;
-    int i_tracks;
     
-    _obj->pp_lba = malloc( (_obj->num_tracks + 1) * sizeof(int) );
-    if( _obj->pp_lba == NULL )
+    CDTOCDescriptor *pTrackDescriptors;
+    track_t i_track;
+    
+    env->pp_lba = malloc( env->i_descriptors * sizeof(int) );
+    if( env->pp_lba == NULL )
       {
-	cdio_error("Out of memory in allocating track starting LSNs" );
-	free( _obj->pTOC );
+	cdio_warn("Out of memory in allocating track starting LSNs" );
+	free( env->pTOC );
 	return false;
       }
     
-    pTrackDescriptors = _obj->pTOC->descriptors;
-    
-    for( i_tracks = 0, i = 0; i <= _obj->i_descriptors; i++ )
-      {
-	track = pTrackDescriptors[i].point;
+    pTrackDescriptors = env->pTOC->descriptors;
 
-	if( track == 0xA2 )
-	  /* Note leadout should be 0xAA, But OSX seems to use 0xA2. */
+    env->i_first_track = CDIO_CD_MAX_TRACKS+1;
+    env->i_last_track  = CDIO_CD_MIN_TRACK_NO-1;
+    
+    for( i = 0; i <= env->i_descriptors; i++ )
+      {
+	i_track = pTrackDescriptors[i].point;
+
+	cdio_debug( "point: %d, tno: %d, session: %d, adr: %d, control:%d, "
+		   "address: %d:%d:%d, p: %d:%d:%d", 
+		   pTrackDescriptors[i].point,
+		   pTrackDescriptors[i].tno, pTrackDescriptors[i].session,
+		   pTrackDescriptors[i].adr, pTrackDescriptors[i].control,
+		   pTrackDescriptors[i].address.minute,
+		   pTrackDescriptors[i].address.second,
+		   pTrackDescriptors[i].address.frame, 
+		   pTrackDescriptors[i].p.minute,
+		   pTrackDescriptors[i].p.second, 
+		   pTrackDescriptors[i].p.frame );
+
+	/* Skip information where session is 0. */
+	if ( 0 == pTrackDescriptors[i].session ) 
+	  continue;
+
+	if( i_track == OSX_CDROM_LEADOUT_TRACK )
 	  i_leadout = i;
-	
-	if( track > CDIO_CD_MAX_TRACKS || track < CDIO_CD_MIN_TRACK_NO )
+
+	if( i_track > CDIO_CD_MAX_TRACKS || i_track < CDIO_CD_MIN_TRACK_NO )
 	  continue;
 	
-	_obj->pp_lba[i_tracks++] =
+
+	if (env->i_first_track > i_track) 
+	  env->i_first_track = i_track;
+	
+	if (env->i_last_track < i_track) 
+	  env->i_last_track = i_track;
+	
+      }
+
+    /* Now that we know what the first track number is, we can make sure
+       index positions are ordered starting at 0.
+     */
+    for( i = 0; i <= env->i_descriptors; i++ )
+      {
+	i_track = pTrackDescriptors[i].point;
+
+	if( i_track > CDIO_CD_MAX_TRACKS || i_track < CDIO_CD_MIN_TRACK_NO )
+	  continue;
+
+	/* Note what OSX calls a LBA we call an LSN. So below re we 
+	   really have have MSF -> LSN -> LBA.
+	 */
+	env->pp_lba[i_track - env->i_first_track] =
 	  cdio_lsn_to_lba(CDConvertMSFToLBA( pTrackDescriptors[i].p ));
       }
     
     if( i_leadout == -1 )
       {
-	cdio_error( "CD leadout not found" );
-	free( _obj->pp_lba );
-	free( (void *) _obj->pTOC );
+	cdio_warn( "CD leadout not found" );
+	free( env->pp_lba );
+	free( (void *) env->pTOC );
 	return false;
       }
     
-    /* set leadout sector */
-    _obj->pp_lba[i_tracks] =
+    /* Set leadout sector. 
+       Note what OSX calls a LBA we call an LSN. So below re we 
+       really have have MSF -> LSN -> LBA.
+    */
+    env->pp_lba[TOTAL_TRACKS] =
       cdio_lsn_to_lba(CDConvertMSFToLBA( pTrackDescriptors[i_leadout].p ));
   }
 
-  _obj->toc_init   = true;
+  env->toc_init   = true;
 
   return( true ); 
 
@@ -442,24 +474,24 @@ _cdio_read_toc (_img_private_t *_obj)
 
 /*!  
   Return the starting LSN track number
-  track_num in obj.  Track numbers start at 1.
+  i_track in obj.  Track numbers start at 1.
   The "leadout" track is specified either by
-  using track_num LEADOUT_TRACK or the total tracks+1.
+  using i_track LEADOUT_TRACK or the total tracks+1.
   False is returned if there is no track entry.
 */
 static lsn_t
-_cdio_get_track_lba(void *env, track_t track_num)
+_get_track_lba_osx(void *user_data, track_t i_track)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
-  if (!_obj->toc_init) _cdio_read_toc (_obj) ;
+  if (!env->toc_init) _cdio_read_toc (env) ;
 
-  if (track_num == CDIO_CDROM_LEADOUT_TRACK) track_num = TOTAL_TRACKS+1;
+  if (i_track == CDIO_CDROM_LEADOUT_TRACK) i_track = env->i_last_track+1;
 
-  if (track_num > TOTAL_TRACKS+1 || track_num == 0) {
+  if (i_track > env->i_last_track + 1 || i_track < env->i_first_track) {
     return CDIO_INVALID_LSN;
   } else {
-    return _obj->pp_lba[track_num-1];
+    return env->pp_lba[i_track - env->i_first_track];
   }
 }
 
@@ -474,18 +506,18 @@ _cdio_get_track_lba(void *env, track_t track_num)
  */
 
 static int 
-_cdio_eject_media (void *env) {
+_eject_media_osx (void *user_data) {
 
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   FILE *p_eject;
   char *psz_disk;
   char sz_cmd[32];
 
-  if( ( psz_disk = (char *)strstr( _obj->gen.source_name, "disk" ) ) != NULL &&
+  if( ( psz_disk = (char *)strstr( env->gen.source_name, "disk" ) ) != NULL &&
       strlen( psz_disk ) > 4 )
     {
-#define EJECT_CMD "/usr/sbin/disktool -e %s 0"
+#define EJECT_CMD "/usr/sbin/hdiutil eject %s"
       snprintf( sz_cmd, sizeof(sz_cmd), EJECT_CMD, psz_disk );
 #undef EJECT_CMD
       
@@ -518,23 +550,23 @@ _cdio_eject_media (void *env) {
    Return the size of the CD in logical block address (LBA) units.
  */
 static uint32_t 
-_cdio_stat_size (void *env)
+_stat_size_osx (void *user_data)
 {
-  return _cdio_get_track_lba(env, CDIO_CDROM_LEADOUT_TRACK);
+  return _get_track_lba_osx(user_data, CDIO_CDROM_LEADOUT_TRACK);
 }
 
 /*!
   Return the value associated with the key "arg".
 */
 static const char *
-_cdio_get_arg (void *env, const char key[])
+_get_arg_osx (void *user_data, const char key[])
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   if (!strcmp (key, "source")) {
-    return _obj->gen.source_name;
+    return env->gen.source_name;
   } else if (!strcmp (key, "access-mode")) {
-    switch (_obj->access_mode) {
+    switch (env->access_mode) {
     case _AM_OSX:
       return "OS X";
     case _AM_NONE:
@@ -549,45 +581,28 @@ _cdio_get_arg (void *env, const char key[])
   CDIO_INVALID_TRACK is returned on error.
 */
 static track_t
-_cdio_get_first_track_num(void *env) 
+_get_first_track_num_osx(void *user_data) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->toc_init) _cdio_read_toc (_obj) ;
-  
-  {
-    track_t track = CDIO_INVALID_TRACK;
-    int i;
-    CDTOCDescriptor *pTrackDescriptors;
+  if (!env->toc_init) _cdio_read_toc (env) ;
 
-    pTrackDescriptors = _obj->pTOC->descriptors;
-
-    for( i = 0; i < _obj->i_descriptors; i++ )
-    {
-        track = pTrackDescriptors[i].point;
-
-	if( track > CDIO_CD_MAX_TRACKS || track < CDIO_CD_MIN_TRACK_NO )
-	  continue;
-        return ( track );
-    }
-  }
-  
-  return CDIO_INVALID_TRACK;
+  return env->i_first_track;
 }
 
 /*!
   Return the media catalog number MCN.
  */
 static char *
-_cdio_get_mcn (void *env) {
-  _img_private_t *_obj = env;
+_get_mcn_osx (const void *user_data) {
+  const _img_private_t *env = user_data;
   dk_cd_read_mcn_t cd_read;
 
   memset( &cd_read, 0, sizeof(cd_read) );
 
-  if( ioctl( _obj->gen.fd, DKIOCCDREADMCN, &cd_read ) < 0 )
+  if( ioctl( env->gen.fd, DKIOCCDREADMCN, &cd_read ) < 0 )
   {
-    cdio_error( "could not read MCN, %s", strerror(errno) );
+    cdio_debug( "could not read MCN, %s", strerror(errno) );
     return NULL;
   }
   return strdup((char*)cd_read.mcn);
@@ -600,46 +615,56 @@ _cdio_get_mcn (void *env) {
   This is the externally called interface.
 */
 static track_t
-_cdio_get_num_tracks(void *env) 
+_get_num_tracks_osx(void *user_data) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->toc_init) _cdio_read_toc (_obj) ;
-  return( _obj->num_tracks );
+  if (!env->toc_init) _cdio_read_toc (env) ;
+
+  return( TOTAL_TRACKS );
 }
 
 /*!  
   Get format of track. 
 */
 static track_format_t
-_cdio_get_track_format(void *env, track_t track_num) 
+_get_track_format_osx(void *user_data, track_t i_track) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
+  dk_cd_read_track_info_t cd_read;
   CDTrackInfo a_track;
-  
-  if (!_obj->toc_init) _cdio_read_toc (_obj) ;
 
-  if (track_num > TOTAL_TRACKS || track_num == 0)
+  if (i_track > env->i_last_track || i_track < env->i_first_track)
     return TRACK_FORMAT_ERROR;
     
-  dk_cd_read_track_info_t cd_read;
   memset( &cd_read, 0, sizeof(cd_read) );
 
-  cd_read.address = track_num;
+  cd_read.address = i_track;
   cd_read.addressType = kCDTrackInfoAddressTypeTrackNumber;
   
   cd_read.buffer = &a_track;
   cd_read.bufferLength = sizeof(CDTrackInfo);
   
-  if( ioctl( _obj->gen.fd, DKIOCCDREADTRACKINFO, &cd_read ) == -1 )
+  if( ioctl( env->gen.fd, DKIOCCDREADTRACKINFO, &cd_read ) == -1 )
   {
-    cdio_error( "could not read trackinfo for track %d", track_num );
-    return -1;
+    cdio_warn( "could not read trackinfo for track %d", i_track );
+    return TRACK_FORMAT_ERROR;
   }
 
-  cdio_warn( "trackinfo trackMode: %x dataMode: %x", a_track.trackMode, a_track.dataMode );
+  cdio_debug( "%d: trackinfo trackMode: %x dataMode: %x", i_track, a_track.trackMode, a_track.dataMode );
+
+  if (a_track.trackMode == CDIO_CDROM_DATA_TRACK) {
+    if (a_track.dataMode == CDROM_CDI_TRACK) {
+      return TRACK_FORMAT_CDI;
+    } else if (a_track.dataMode == CDROM_XA_TRACK) {
+      return TRACK_FORMAT_XA;
+    } else {
+      return TRACK_FORMAT_DATA;
+    }
+  } else {
+    return TRACK_FORMAT_AUDIO;
+  }
 
-  return TRACK_FORMAT_AUDIO;
 }
 
 /*!
@@ -651,25 +676,34 @@ _cdio_get_track_format(void *env, track_t track_num)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *env, track_t track_num) 
+_get_track_green_osx(void *user_data, track_t i_track) 
 {
+  _img_private_t *env = user_data;
+  CDTrackInfo a_track;
 
-#if 0  
-  if (!_obj->toc_init) _cdio_read_toc (_obj) ;
+  if (!env->toc_init) _cdio_read_toc (env) ;
 
-  if (track_num == CDIO_LEADOUT_TRACK) track_num = TOTAL_TRACKS+1;
-
-  if (track_num > TOTAL_TRACKS+1 || track_num == 0)
+  if ( i_track > env->i_last_track || i_track < env->i_first_track )
     return false;
 
-  /* FIXME: Dunno if this is the right way, but it's what 
-     I was using in cdinfo for a while.
-   */
-  return ((_obj->tocent[track_num-1].cdte_ctrl & 2) != 0);
-#else 
-  /* FIXME! Figure out how to do. */
-  return true;
-#endif
+  else {
+
+    dk_cd_read_track_info_t cd_read;
+    
+    memset( &cd_read, 0, sizeof(cd_read) );
+    
+    cd_read.address      = i_track;
+    cd_read.addressType  = kCDTrackInfoAddressTypeTrackNumber;
+    
+    cd_read.buffer       = &a_track;
+    cd_read.bufferLength = sizeof(CDTrackInfo);
+    
+    if( ioctl( env->gen.fd, DKIOCCDREADTRACKINFO, &cd_read ) == -1 ) {
+      cdio_warn( "could not read trackinfo for track %d", i_track );
+      return false;
+    }
+    return ((a_track.trackMode & CDIO_CDROM_DATA_TRACK) != 0);
+  }
 }
 
 #endif /* HAVE_DARWIN_CDROM */
@@ -694,16 +728,16 @@ cdio_get_devices_osx(void)
   kern_result = IOMasterPort( MACH_PORT_NULL, &master_port );
   if( kern_result != KERN_SUCCESS )
     {
-      return( nil );
+      return( NULL );
     }
   
   classes_to_match = IOServiceMatching( kIOCDMediaClass );
   if( classes_to_match == NULL )
     {
-      return( nil );
+      return( NULL );
     }
   
-  CFDictionarySetValue( classes_to_match, CFSTR(kIOMediaEjectable),
+  CFDictionarySetValue( classes_to_match, CFSTR(kIOMediaEjectableKey),
 			kCFBooleanTrue );
   
   kern_result = IOServiceGetMatchingServices( master_port, 
@@ -711,7 +745,7 @@ cdio_get_devices_osx(void)
 					      &media_iterator );
   if( kern_result != KERN_SUCCESS )
     {
-      return( nil );
+      return( NULL );
     }
   
   next_media = IOIteratorNext( media_iterator );
@@ -724,7 +758,7 @@ cdio_get_devices_osx(void)
       do
 	{
 	  str_bsd_path = IORegistryEntryCreateCFProperty( next_media,
-							  CFSTR( kIOBSDName ),
+							  CFSTR( kIOBSDNameKey ),
 							  kCFAllocatorDefault,
 							  0 );
 	  if( str_bsd_path == NULL )
@@ -776,16 +810,16 @@ cdio_get_default_device_osx(void)
   kern_result = IOMasterPort( MACH_PORT_NULL, &master_port );
   if( kern_result != KERN_SUCCESS )
     {
-      return( nil );
+      return( NULL );
     }
   
   classes_to_match = IOServiceMatching( kIOCDMediaClass );
   if( classes_to_match == NULL )
     {
-      return( nil );
+      return( NULL );
     }
   
-  CFDictionarySetValue( classes_to_match, CFSTR(kIOMediaEjectable),
+  CFDictionarySetValue( classes_to_match, CFSTR(kIOMediaEjectableKey),
 			kCFBooleanTrue );
   
   kern_result = IOServiceGetMatchingServices( master_port, 
@@ -793,7 +827,7 @@ cdio_get_default_device_osx(void)
 					      &media_iterator );
   if( kern_result != KERN_SUCCESS )
     {
-      return( nil );
+      return( NULL );
     }
   
   next_media = IOIteratorNext( media_iterator );
@@ -806,7 +840,7 @@ cdio_get_default_device_osx(void)
       do
 	{
 	  str_bsd_path = IORegistryEntryCreateCFProperty( next_media,
-							  CFSTR( kIOBSDName ),
+							  CFSTR( kIOBSDNameKey ),
 							  kCFAllocatorDefault,
 							  0 );
 	  if( str_bsd_path == NULL )
@@ -845,36 +879,53 @@ cdio_get_default_device_osx(void)
   ones to set that up.
  */
 CdIo *
-cdio_open_osx (const char *orig_source_name)
+cdio_open_am_osx (const char *psz_source_name, const char *psz_access_mode)
+{
+
+  if (psz_access_mode != NULL)
+    cdio_warn ("there is only one access mode for OS X. Arg %s ignored",
+	       psz_access_mode);
+  return cdio_open_osx(psz_source_name);
+}
+
+
+/*!
+  Initialization routine. This is the only thing that doesn't
+  get called via a function pointer. In fact *we* are the
+  ones to set that up.
+ */
+CdIo *
+cdio_open_osx (const char *psz_orig_source)
 {
 
 #ifdef HAVE_DARWIN_CDROM
   CdIo *ret;
   _img_private_t *_data;
-  char *source_name;
+  char *psz_source;
 
   cdio_funcs _funcs = {
-    .eject_media        = _cdio_eject_media,
-    .free               = _cdio_osx_free,
-    .get_arg            = _cdio_get_arg,
+    .eject_media        = _eject_media_osx,
+    .free               = _free_osx,
+    .get_arg            = _get_arg_osx,
     .get_default_device = cdio_get_default_device_osx,
     .get_devices        = cdio_get_devices_osx,
-    .get_first_track_num= _cdio_get_first_track_num,
-    .get_mcn            = _cdio_get_mcn,
-    .get_num_tracks     = _cdio_get_num_tracks,
-    .get_track_format   = _cdio_get_track_format,
-    .get_track_green    = _cdio_get_track_green,
-    .get_track_lba      = _cdio_get_track_lba,
+    .get_drive_cap      = NULL,
+    .get_first_track_num= _get_first_track_num_osx,
+    .get_mcn            = _get_mcn_osx,
+    .get_num_tracks     = _get_num_tracks_osx,
+    .get_track_format   = _get_track_format_osx,
+    .get_track_green    = _get_track_green_osx,
+    .get_track_lba      = _get_track_lba_osx,
     .get_track_msf      = NULL,
     .lseek              = cdio_generic_lseek,
     .read               = cdio_generic_read,
-    .read_audio_sectors = _cdio_read_audio_sectors,
-    .read_mode1_sector  = _cdio_read_mode1_sector,
-    .read_mode1_sectors = _cdio_read_mode1_sectors,
-    .read_mode2_sector  = _cdio_read_mode2_sector,
-    .read_mode2_sectors = _cdio_read_mode2_sectors,
-    .set_arg            = _cdio_set_arg,
-    .stat_size          = _cdio_stat_size
+    .read_audio_sectors = _get_read_audio_sectors_osx,
+    .read_mode1_sector  = _get_read_mode1_sector_osx,
+    .read_mode1_sectors = _get_read_mode1_sectors_osx,
+    .read_mode2_sector  = _get_read_mode2_sector_osx,
+    .read_mode2_sectors = _get_read_mode2_sectors_osx,
+    .set_arg            = _set_arg_osx,
+    .stat_size          = _stat_size_osx
   };
 
   _data                 = _cdio_malloc (sizeof (_img_private_t));
@@ -882,13 +933,22 @@ cdio_open_osx (const char *orig_source_name)
   _data->gen.init       = false;
   _data->gen.fd         = -1;
 
-  if (NULL == orig_source_name) {
-    source_name=cdio_get_default_device_linux();
-    if (NULL == source_name) return NULL;
-    _cdio_set_arg(_data, "source", source_name);
-    free(source_name);
-  } else 
-    _cdio_set_arg(_data, "source", orig_source_name);
+  if (NULL == psz_orig_source) {
+    psz_source=cdio_get_default_device_osx();
+    if (NULL == psz_source) return NULL;
+    _set_arg_osx(_data, "source", psz_source);
+    free(psz_source);
+  } else {
+    if (cdio_is_device_generic(psz_orig_source))
+      _set_arg_osx(_data, "source", psz_orig_source);
+    else {
+      /* The below would be okay if all device drivers worked this way. */
+#if 0
+      cdio_info ("source %s is a not a device", psz_orig_source);
+#endif
+      return NULL;
+    }
+  }
 
   ret = cdio_new (_data, &_funcs);
   if (ret == NULL) return NULL;
