@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_sunos.c,v 1.25 2004/03/16 12:18:32 rocky Exp $
+    $Id: _cdio_sunos.c,v 1.40 2004/06/25 21:10:44 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -26,22 +26,23 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_GLOB_H
-#include <glob.h>
-#endif
 
 #include <cdio/logging.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
+#include <cdio/scsi_mmc.h>
 #include "cdio_assert.h"
 #include "cdio_private.h"
-#include "scsi_mmc.h"
 
 #define DEFAULT_CDIO_DEVICE "/vol/dev/aliases/cdrom0"
 
 #ifdef HAVE_SOLARIS_CDROM
 
-static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.25 2004/03/16 12:18:32 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.40 2004/06/25 21:10:44 rocky Exp $";
+
+#ifdef HAVE_GLOB_H
+#include <glob.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -63,17 +64,12 @@ static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.25 2004/03/16 12:18:32 rock
 #include <sys/types.h>
 #include <sys/ioctl.h>
 
-#define TOTAL_TRACKS    (_obj->tochdr.cdth_trk1)
-#define FIRST_TRACK_NUM (_obj->tochdr.cdth_trk0)
+#define TOTAL_TRACKS    (env->tochdr.cdth_trk1)
+#define FIRST_TRACK_NUM (env->tochdr.cdth_trk0)
 
 /* reader */
 
-typedef struct {
-  /* Things common to all drivers like this. 
-     This must be first. */
-  generic_img_private_t gen; 
-  
-  enum {
+typedef  enum {
     _AM_NONE,
     _AM_SUN_CTRL_ATAPI,
     _AM_SUN_CTRL_SCSI
@@ -81,25 +77,51 @@ typedef struct {
     _AM_READ_CD,
     _AM_READ_10
 #endif
-  } access_mode;
+} access_mode_t;
 
 
+typedef struct {
+  /* Things common to all drivers like this. 
+     This must be first. */
+  generic_img_private_t gen; 
+  
+  access_mode_t access_mode;
+
   /* Track information */
   struct cdrom_tochdr    tochdr;
   struct cdrom_tocentry  tocent[100];    /* entry info for each track */
 
 } _img_private_t;
 
+static access_mode_t 
+str_to_access_mode_sunos(const char *psz_access_mode) 
+{
+  const access_mode_t default_access_mode = _AM_SUN_CTRL_SCSI;
+
+  if (NULL==psz_access_mode) return default_access_mode;
+  
+  if (!strcmp(psz_access_mode, "ATAPI"))
+    return _AM_SUN_CTRL_SCSI; /* force ATAPI to be SCSI */
+  else if (!strcmp(psz_access_mode, "SCSI"))
+    return _AM_SUN_CTRL_SCSI;
+  else {
+    cdio_warn ("unknown access type: %s. Default SCSI used.", 
+	       psz_access_mode);
+    return default_access_mode;
+  }
+}
+
+
 /*!
   Initialize CD device.
  */
 static bool
-_cdio_init (_img_private_t *_obj)
+_cdio_init (_img_private_t *env)
 {
 
-  if (!cdio_generic_init(_obj)) return false;
+  if (!cdio_generic_init(env)) return false;
   
-  _obj->access_mode = _AM_SUN_CTRL_SCSI;    
+  env->access_mode = _AM_SUN_CTRL_SCSI;    
 
   return true;
 }
@@ -113,7 +135,7 @@ _cdio_init (_img_private_t *_obj)
 */
 
 static int
-_cdio_read_audio_sectors (void *env, void *data, lsn_t lsn, 
+_cdio_read_audio_sectors (void *user_data, void *data, lsn_t lsn, 
 			  unsigned int nblocks)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
@@ -121,31 +143,31 @@ _cdio_read_audio_sectors (void *env, void *data, lsn_t lsn,
   msf_t _msf;
   struct cdrom_cdda cdda;
 
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
   msf->cdmsf_min0   = from_bcd8(_msf.m);
   msf->cdmsf_sec0   = from_bcd8(_msf.s);
   msf->cdmsf_frame0 = from_bcd8(_msf.f);
   
-  if (_obj->gen.ioctls_debugged == 75)
+  if (env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
   
-  if (_obj->gen.ioctls_debugged == 30 * 75)
+  if (env->gen.ioctls_debugged == 30 * 75)
     cdio_debug ("only displaying every 30*75th ioctl from now on");
   
-  if (_obj->gen.ioctls_debugged < 75 
-      || (_obj->gen.ioctls_debugged < (30 * 75)  
-	  && _obj->gen.ioctls_debugged % 75 == 0)
-      || _obj->gen.ioctls_debugged % (30 * 75) == 0)
+  if (env->gen.ioctls_debugged < 75 
+      || (env->gen.ioctls_debugged < (30 * 75)  
+	  && env->gen.ioctls_debugged % 75 == 0)
+      || env->gen.ioctls_debugged % (30 * 75) == 0)
     cdio_debug ("reading %d", lsn);
   
-  _obj->gen.ioctls_debugged++;
+  env->gen.ioctls_debugged++;
   
   cdda.cdda_addr   = lsn;
   cdda.cdda_length = nblocks;
   cdda.cdda_data   = (caddr_t) data;
-  if (ioctl (_obj->gen.fd, CDROMCDDA, &cdda) == -1) {
+  if (ioctl (env->gen.fd, CDROMCDDA, &cdda) == -1) {
     perror ("ioctl(..,CDROMCDDA,..)");
 	return 1;
 	/* exit (EXIT_FAILURE); */
@@ -183,16 +205,16 @@ _cdio_read_mode1_sector (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn, 
+_cdio_read_mode1_sectors (void *user_data, void *data, lsn_t lsn, 
 			  bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   unsigned int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _cdio_read_mode1_sector (_obj, 
+    if ( (retval = _cdio_read_mode1_sector (env, 
 					    ((char *)data) + (blocksize * i),
 					    lsn + i, b_form2)) )
       return retval;
@@ -205,7 +227,7 @@ _cdio_read_mode1_sectors (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sector (void *env, void *data, lsn_t lsn, 
+_cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn, 
 			 bool b_form2)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
@@ -214,27 +236,27 @@ _cdio_read_mode2_sector (void *env, void *data, lsn_t lsn,
   int offset = 0;
   struct cdrom_cdxa cd_read;
 
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
   msf->cdmsf_min0 = from_bcd8(_msf.m);
   msf->cdmsf_sec0 = from_bcd8(_msf.s);
   msf->cdmsf_frame0 = from_bcd8(_msf.f);
   
-  if (_obj->gen.ioctls_debugged == 75)
+  if (env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
   
-  if (_obj->gen.ioctls_debugged == 30 * 75)
+  if (env->gen.ioctls_debugged == 30 * 75)
     cdio_debug ("only displaying every 30*75th ioctl from now on");
   
-  if (_obj->gen.ioctls_debugged < 75 
-      || (_obj->gen.ioctls_debugged < (30 * 75)  
-	  && _obj->gen.ioctls_debugged % 75 == 0)
-      || _obj->gen.ioctls_debugged % (30 * 75) == 0)
+  if (env->gen.ioctls_debugged < 75 
+      || (env->gen.ioctls_debugged < (30 * 75)  
+	  && env->gen.ioctls_debugged % 75 == 0)
+      || env->gen.ioctls_debugged % (30 * 75) == 0)
     cdio_debug ("reading %2.2d:%2.2d:%2.2d",
 	       msf->cdmsf_min0, msf->cdmsf_sec0, msf->cdmsf_frame0);
   
-  _obj->gen.ioctls_debugged++;
+  env->gen.ioctls_debugged++;
   
   /* Using CDROMXA ioctl will actually use the same uscsi command
    * as ATAPI, except we don't need to be root
@@ -244,7 +266,7 @@ _cdio_read_mode2_sector (void *env, void *data, lsn_t lsn,
   cd_read.cdxa_data = buf;
   cd_read.cdxa_length = 1;
   cd_read.cdxa_format = CDROM_XA_SECTOR_DATA;
-  if (ioctl (_obj->gen.fd, CDROMCDXA, &cd_read) == -1) {
+  if (ioctl (env->gen.fd, CDROMCDXA, &cd_read) == -1) {
     perror ("ioctl(..,CDROMCDXA,..)");
     return 1;
     /* exit (EXIT_FAILURE); */
@@ -264,16 +286,16 @@ _cdio_read_mode2_sector (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn, 
+_cdio_read_mode2_sectors (void *user_data, void *data, lsn_t lsn, 
 			  bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   unsigned int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _cdio_read_mode2_sector (_obj, 
+    if ( (retval = _cdio_read_mode2_sector (env, 
 					    ((char *)data) + (blocksize * i),
 					    lsn + i, b_form2)) )
       return retval;
@@ -286,16 +308,16 @@ _cdio_read_mode2_sectors (void *env, void *data, lsn_t lsn,
    Return the size of the CD in logical block address (LBA) units.
  */
 static uint32_t 
-_cdio_stat_size (void *env)
+_cdio_stat_size (void *user_data)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   struct cdrom_tocentry tocent;
   uint32_t size;
 
   tocent.cdte_track = CDIO_CDROM_LEADOUT_TRACK;
   tocent.cdte_format = CDROM_LBA;
-  if (ioctl (_obj->gen.fd, CDROMREADTOCENTRY, &tocent) == -1)
+  if (ioctl (env->gen.fd, CDROMREADTOCENTRY, &tocent) == -1)
     {
       perror ("ioctl(CDROMREADTOCENTRY)");
       exit (EXIT_FAILURE);
@@ -315,27 +337,22 @@ _cdio_stat_size (void *env)
   0 is returned if no error was found, and nonzero if there as an error.
 */
 static int
-_cdio_set_arg (void *env, const char key[], const char value[])
+_set_arg_solaris (void *user_data, const char key[], const char value[])
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   if (!strcmp (key, "source"))
     {
       if (!value)
 	return -2;
 
-      free (_obj->gen.source_name);
+      free (env->gen.source_name);
       
-      _obj->gen.source_name = strdup (value);
+      env->gen.source_name = strdup (value);
     }
   else if (!strcmp (key, "access-mode"))
     {
-      if (!strcmp(value, "ATAPI"))
-	_obj->access_mode = _AM_SUN_CTRL_SCSI; /* force ATAPI to be SCSI */
-      else if (!strcmp(value, "SCSI"))
-	_obj->access_mode = _AM_SUN_CTRL_SCSI;
-      else
-	cdio_warn ("unknown access type: %s. ignored.", value);
+      env->access_mode = str_to_access_mode_sunos(key);
     }
   else 
     return -1;
@@ -348,23 +365,23 @@ _cdio_set_arg (void *env, const char key[], const char value[])
   Return true if successful or false if an error.
 */
 static bool
-_cdio_read_toc (_img_private_t *_obj) 
+_cdio_read_toc (_img_private_t *env) 
 {
   int i;
 
   /* read TOC header */
-  if ( ioctl(_obj->gen.fd, CDROMREADTOCHDR, &_obj->tochdr) == -1 ) {
-    cdio_error("%s: %s\n", 
+  if ( ioctl(env->gen.fd, CDROMREADTOCHDR, &env->tochdr) == -1 ) {
+    cdio_warn("%s: %s\n", 
             "error in ioctl CDROMREADTOCHDR", strerror(errno));
     return false;
   }
 
   /* read individual tracks */
-  for (i=_obj->tochdr.cdth_trk0; i<=_obj->tochdr.cdth_trk1; i++) {
-    _obj->tocent[i-1].cdte_track = i;
-    _obj->tocent[i-1].cdte_format = CDROM_MSF;
-    if ( ioctl(_obj->gen.fd, CDROMREADTOCENTRY, &_obj->tocent[i-1]) == -1 ) {
-      cdio_error("%s %d: %s\n",
+  for (i=env->tochdr.cdth_trk0; i<=env->tochdr.cdth_trk1; i++) {
+    env->tocent[i-1].cdte_track = i;
+    env->tocent[i-1].cdte_format = CDROM_MSF;
+    if ( ioctl(env->gen.fd, CDROMREADTOCENTRY, &env->tocent[i-1]) == -1 ) {
+      cdio_warn("%s %d: %s\n",
               "error in ioctl CDROMREADTOCENTRY for track", 
               i, strerror(errno));
       return false;
@@ -372,18 +389,18 @@ _cdio_read_toc (_img_private_t *_obj)
   }
 
   /* read the lead-out track */
-  _obj->tocent[_obj->tochdr.cdth_trk1].cdte_track = CDIO_CDROM_LEADOUT_TRACK;
-  _obj->tocent[_obj->tochdr.cdth_trk1].cdte_format = CDROM_MSF;
+  env->tocent[env->tochdr.cdth_trk1].cdte_track = CDIO_CDROM_LEADOUT_TRACK;
+  env->tocent[env->tochdr.cdth_trk1].cdte_format = CDROM_MSF;
 
-  if (ioctl(_obj->gen.fd, CDROMREADTOCENTRY, 
-	    &_obj->tocent[_obj->tochdr.cdth_trk1]) == -1 ) {
-    cdio_error("%s: %s\n", 
+  if (ioctl(env->gen.fd, CDROMREADTOCENTRY, 
+	    &env->tocent[env->tochdr.cdth_trk1]) == -1 ) {
+    cdio_warn("%s: %s\n", 
 	     "error in ioctl CDROMREADTOCENTRY for lead-out",
             strerror(errno));
     return false;
   }
 
-  _obj->gen.toc_init = true;
+  env->gen.toc_init = true;
   return true;
 }
 
@@ -392,17 +409,17 @@ _cdio_read_toc (_img_private_t *_obj)
   also free obj.
  */
 static int 
-_cdio_eject_media (void *env) {
+_cdio_eject_media (void *user_data) {
 
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   int ret;
 
-  close(_obj->gen.fd);
-  _obj->gen.fd = -1;
-  if (_obj->gen.fd > -1) {
-    if ((ret = ioctl(_obj->gen.fd, CDROMEJECT)) != 0) {
-      cdio_generic_free((void *) _obj);
-      cdio_error ("CDROMEJECT failed: %s\n", strerror(errno));
+  close(env->gen.fd);
+  env->gen.fd = -1;
+  if (env->gen.fd > -1) {
+    if ((ret = ioctl(env->gen.fd, CDROMEJECT)) != 0) {
+      cdio_generic_free((void *) env);
+      cdio_warn ("CDROMEJECT failed: %s\n", strerror(errno));
       return 1;
     } else {
       return 0;
@@ -419,7 +436,7 @@ _cdio_malloc_and_zero(size_t size) {
   if( !size ) size++;
     
   if((ptr = malloc(size)) == NULL) {
-    cdio_error("malloc() failed: %s", strerror(errno));
+    cdio_warn("malloc() failed: %s", strerror(errno));
     return NULL;
   }
 
@@ -431,14 +448,14 @@ _cdio_malloc_and_zero(size_t size) {
   Return the value associated with the key "arg".
 */
 static const char *
-_cdio_get_arg (void *env, const char key[])
+_cdio_get_arg (void *user_data, const char key[])
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   if (!strcmp (key, "source")) {
-    return _obj->gen.source_name;
+    return env->gen.source_name;
   } else if (!strcmp (key, "access-mode")) {
-    switch (_obj->access_mode) {
+    switch (env->access_mode) {
     case _AM_SUN_CTRL_ATAPI:
       return "ATAPI";
     case _AM_SUN_CTRL_SCSI:
@@ -481,30 +498,147 @@ cdio_get_default_device_solaris(void)
   return strdup(DEFAULT_CDIO_DEVICE);
 }
 
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+static cdio_drive_cap_t
+_cdio_get_drive_cap_solaris (const void *user_data)
+{
+#if 1
+  struct uscsi_cmd my_cmd;
+  int32_t i_drivetype = 0;
+  char buf[192] = { 0, };
+  unsigned char my_rq_buf[26];
+  unsigned char my_scsi_cdb[6];
+  const _img_private_t *env = user_data;
+  int rc;
+  
+  memset(&my_scsi_cdb, 0, sizeof(my_scsi_cdb));
+  memset(&my_rq_buf,   0, sizeof(my_rq_buf));
+  
+  /* Initialize my_scsi_cdb as a Mode Select(6) */
+  CDIO_MMC_SET_COMMAND(my_scsi_cdb, CDIO_MMC_MODE_SENSE);
+  my_scsi_cdb[1] = 0x0;  
+  my_scsi_cdb[2] = 0x2a; /* MODE_PAGE_CAPABILITIES*/;
+  my_scsi_cdb[3] = 0;    /* Not used */
+  my_scsi_cdb[4] = 128; 
+  my_scsi_cdb[5] = 0;    /* Not used */
+  
+  my_cmd.uscsi_flags = (USCSI_READ|USCSI_RQENABLE);  /* We're going get data */
+  my_cmd.uscsi_timeout = 15;             /* Allow 15 seconds for completion */
+  my_cmd.uscsi_cdb = my_scsi_cdb;        /* We'll be using the array above for the CDB */
+  my_cmd.uscsi_bufaddr = buf;   /* The block and page data is stored here */
+  my_cmd.uscsi_buflen = sizeof(buf); 
+  my_cmd.uscsi_cdblen = 6;               /* The CDB is 6 bytes long */
+  my_cmd.uscsi_rqlen =  24;              /* The request sense buffer (only valid on a check condition) is 26 bytes long */
+  my_cmd.uscsi_rqbuf = my_rq_buf;        /* Pointer to the request sense buffer */
+  
+  rc = ioctl(env->gen.fd, USCSICMD, &my_cmd);
+  if(rc == 0) {
+    unsigned int n=buf[3]+4;
+    /* Reader? */
+    if (buf[n+5] & 0x01) i_drivetype |= CDIO_DRIVE_CAP_CD_AUDIO;
+    if (buf[n+2] & 0x02) i_drivetype |= CDIO_DRIVE_CAP_CD_RW;
+    if (buf[n+2] & 0x08) i_drivetype |= CDIO_DRIVE_CAP_DVD;
+    
+    /* Writer? */
+    if (buf[n+3] & 0x01) i_drivetype |= CDIO_DRIVE_CAP_CD_R;
+    if (buf[n+3] & 0x10) i_drivetype |= CDIO_DRIVE_CAP_DVD_R;
+    if (buf[n+3] & 0x20) i_drivetype |= CDIO_DRIVE_CAP_DVD_RAM;
+    
+    if (buf[n+6] & 0x08) i_drivetype |= CDIO_DRIVE_CAP_OPEN_TRAY;
+    if (buf[n+6] >> 5 != 0) i_drivetype |= CDIO_DRIVE_CAP_CLOSE_TRAY;
+    
+  } else {
+    i_drivetype = CDIO_DRIVE_CAP_CD_AUDIO | CDIO_DRIVE_CAP_UNKNOWN;
+  }
+  return i_drivetype;
+
+#else
+  return CDIO_DRIVE_CAP_UNKNOWN | CDIO_DRIVE_CAP_CD_AUDIO 
+    | CDIO_DRIVE_CAP_CD_RW;
+#endif
+}
+
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+static char *
+_cdio_get_mcn_solaris (const void *user_data)
+{
+#if 0
+  struct uscsi_cmd my_cmd;
+  char buf[192] = { 0, };
+  unsigned char my_rq_buf[32];
+  unsigned char my_scsi_cdb[6];
+  const _img_private_t *env = user_data;
+  int rc;
+  
+  memset(&my_scsi_cdb, 0, sizeof(my_scsi_cdb));
+  memset(&my_rq_buf,   0, sizeof(my_rq_buf));
+  
+  CDIO_MMC_SET_COMMAND(my_scsi_cdb, CDIO_MMC_GPCMD_READ_SUBCHANNEL);
+  my_scsi_cdb[1] = 0x0;  
+  my_scsi_cdb[2] = 0x40; 
+  my_scsi_cdb[3] = 02;    /* Give media catalog number. */
+  my_scsi_cdb[4] = 0;    /* Not used */
+  my_scsi_cdb[5] = 0;    /* Not used */
+  my_scsi_cdb[6] = 0;    /* Not used */
+  my_scsi_cdb[7] = 0;    /* Not used */
+  my_scsi_cdb[8] = 28; 
+  my_scsi_cdb[9] = 0;    /* Not used */
+  
+  my_cmd.uscsi_flags = (USCSI_READ|USCSI_RQENABLE);  /* We're going get data */
+  my_cmd.uscsi_timeout = 15;             /* Allow 15 seconds for completion */
+  my_cmd.uscsi_cdb = my_scsi_cdb;        /* We'll be using the array above for the CDB */
+  my_cmd.uscsi_bufaddr = buf;   /* The block and page data is stored here */
+  my_cmd.uscsi_buflen = sizeof(buf); 
+  my_cmd.uscsi_cdblen = 6;               /* The CDB is 6 bytes long */
+  my_cmd.uscsi_rqlen =  24;              /* The request sense buffer (only valid on a check condition) is 26 bytes long */
+  my_cmd.uscsi_rqbuf = my_rq_buf;        /* Pointer to the request sense buffer */
+  
+  rc = ioctl(env->gen.fd, USCSICMD, &my_cmd);
+  if(rc == 0) {
+    return strdup(&buf[9]);
+  }
+  return NULL;
+#else
+  return NULL;
+#endif
+}
+
+
 /*!
   Return the number of of the first track. 
   CDIO_INVALID_TRACK is returned on error.
 */
 static track_t
-_cdio_get_first_track_num(void *env) 
+_cdio_get_first_track_num(void *user_data) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
+  if (!env->gen.toc_init) _cdio_read_toc (env) ;
 
   return FIRST_TRACK_NUM;
 }
 
+
 /*!
   Return the number of tracks in the current medium.
 */
 static track_t
-_cdio_get_num_tracks(void *env) 
+_cdio_get_num_tracks(void *user_data)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
-
   return TOTAL_TRACKS;
 }
 
@@ -512,23 +646,22 @@ _cdio_get_num_tracks(void *env)
   Get format of track. 
 */
 static track_format_t
-_cdio_get_track_format(void *env, track_t track_num) 
+_cdio_get_track_format(void *user_data, track_t i_track) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.init) _cdio_init(_obj);
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
-
-  if (track_num > TOTAL_TRACKS || track_num == 0)
+  if ( (i_track > TOTAL_TRACKS+FIRST_TRACK_NUM) || i_track < FIRST_TRACK_NUM)
     return TRACK_FORMAT_ERROR;
 
+  i_track -= FIRST_TRACK_NUM;
+
   /* This is pretty much copied from the "badly broken" cdrom_count_tracks
      in linux/cdrom.c.
    */
-  if (_obj->tocent[track_num-1].cdte_ctrl & CDROM_DATA_TRACK) {
-    if (_obj->tocent[track_num-1].cdte_format == 0x10)
+  if (env->tocent[i_track].cdte_ctrl & CDROM_DATA_TRACK) {
+    if (env->tocent[i_track].cdte_format == CDIO_CDROM_CDI_TRACK)
       return TRACK_FORMAT_CDI;
-    else if (_obj->tocent[track_num-1].cdte_format == 0x20) 
+    else if (env->tocent[i_track].cdte_format == CDIO_CDROM_XA_TRACK) 
       return TRACK_FORMAT_XA;
     else
       return TRACK_FORMAT_DATA;
@@ -546,47 +679,50 @@ _cdio_get_track_format(void *env, track_t track_num)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *env, track_t track_num) 
+_cdio_get_track_green(void *user_data, track_t i_track) 
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
   
-  if (!_obj->gen.init) _cdio_init(_obj);
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
+  if (!env->gen.init) _cdio_init(env);
+  if (!env->gen.toc_init) _cdio_read_toc (env) ;
 
-  if (track_num == CDIO_CDROM_LEADOUT_TRACK) track_num = TOTAL_TRACKS+1;
-
-  if (track_num > TOTAL_TRACKS+1 || track_num == 0)
+  if (i_track > TOTAL_TRACKS+FIRST_TRACK_NUM || i_track < FIRST_TRACK_NUM)
     return false;
 
+  i_track -= FIRST_TRACK_NUM;
+
   /* FIXME: Dunno if this is the right way, but it's what 
-     I was using in cdinfo for a while.
+     I was using in cd-info for a while.
    */
-  return ((_obj->tocent[track_num-1].cdte_ctrl & 2) != 0);
+  return ((env->tocent[i_track].cdte_ctrl & 2) != 0);
 }
 
 /*!  
   Return the starting MSF (minutes/secs/frames) for track number
-  track_num in obj.  Tracks numbers start at 1.
+  track_num in obj.  Track numbers usually start at something 
+  greater than 0, usually 1.
+
   The "leadout" track is specified either by
   using track_num LEADOUT_TRACK or the total tracks+1.
   False is returned if there is no entry.
 */
 static bool
-_cdio_get_track_msf(void *env, track_t track_num, msf_t *msf)
+_cdio_get_track_msf(void *user_data, track_t i_track, msf_t *msf)
 {
-  _img_private_t *_obj = env;
+  _img_private_t *env = user_data;
 
   if (NULL == msf) return false;
 
-  if (!_obj->gen.init) _cdio_init(_obj);
-  if (!_obj->gen.toc_init) _cdio_read_toc (_obj) ;
+  if (!env->gen.init) _cdio_init(env);
+  if (!env->gen.toc_init) _cdio_read_toc (env) ;
 
-  if (track_num == CDIO_CDROM_LEADOUT_TRACK) track_num = TOTAL_TRACKS+1;
+  if (i_track == CDIO_CDROM_LEADOUT_TRACK) 
+    i_track = TOTAL_TRACKS + FIRST_TRACK_NUM;
 
-  if (track_num > TOTAL_TRACKS+1 || track_num == 0) {
+  if (i_track > (TOTAL_TRACKS+FIRST_TRACK_NUM) || i_track < FIRST_TRACK_NUM) {
     return false;
   } else {
-    struct cdrom_tocentry *msf0 = &_obj->tocent[track_num-1];
+    struct cdrom_tocentry *msf0 = &env->tocent[i_track-1];
     msf->m = to_bcd8(msf0->cdte_addr.msf.minute);
     msf->s = to_bcd8(msf0->cdte_addr.msf.second);
     msf->f = to_bcd8(msf0->cdte_addr.msf.frame);
@@ -603,6 +739,7 @@ cdio_get_default_device_solaris(void)
 {
   return strdup(DEFAULT_CDIO_DEVICE);
 }
+
 #endif /* HAVE_SOLARIS_CDROM */
 
 /*!
@@ -639,12 +776,24 @@ cdio_get_devices_solaris (void)
   ones to set that up.
  */
 CdIo *
-cdio_open_solaris (const char *source_name)
+cdio_open_solaris (const char *psz_source_name)
+{
+  return cdio_open_am_solaris(psz_source_name, NULL);
+}
+
+/*!
+  Initialization routine. This is the only thing that doesn't
+  get called via a function pointer. In fact *we* are the
+  ones to set that up.
+ */
+CdIo *
+cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
 {
 
 #ifdef HAVE_SOLARIS_CDROM
   CdIo *ret;
   _img_private_t *_data;
+  char *psz_source;
 
   cdio_funcs _funcs = {
     .eject_media        = _cdio_eject_media,
@@ -652,8 +801,9 @@ cdio_open_solaris (const char *source_name)
     .get_arg            = _cdio_get_arg,
     .get_devices        = cdio_get_devices_solaris,
     .get_default_device = cdio_get_default_device_solaris,
+    .get_drive_cap      = _cdio_get_drive_cap_solaris,
     .get_first_track_num= _cdio_get_first_track_num,
-    .get_mcn            = NULL,
+    .get_mcn            = _cdio_get_mcn_solaris,
     .get_num_tracks     = _cdio_get_num_tracks,
     .get_track_format   = _cdio_get_track_format,
     .get_track_green    = _cdio_get_track_green,
@@ -667,14 +817,31 @@ cdio_open_solaris (const char *source_name)
     .read_mode2_sector  = _cdio_read_mode2_sector,
     .read_mode2_sectors = _cdio_read_mode2_sectors,
     .stat_size          = _cdio_stat_size,
-    .set_arg            = _cdio_set_arg
+    .set_arg            = _set_arg_solaris
   };
 
   _data                 = _cdio_malloc (sizeof (_img_private_t));
+
+  _data->access_mode    = _AM_SUN_CTRL_SCSI;
+  _data->gen.init       = false;
   _data->gen.fd         = -1;
 
-  _cdio_set_arg(_data, "source", (NULL == source_name) 
-		? DEFAULT_CDIO_DEVICE: source_name);
+  if (NULL == psz_orig_source) {
+    psz_source=cdio_get_default_device_solaris();
+    if (NULL == psz_source) return NULL;
+    _set_arg_solaris(_data, "source", psz_source);
+    free(psz_source);
+  } else {
+    if (cdio_is_device_generic(psz_orig_source))
+      _set_arg_solaris(_data, "source", psz_orig_source);
+    else {
+      /* The below would be okay if all device drivers worked this way. */
+#if 0
+      cdio_info ("source %s is a not a device", psz_orig_source);
+#endif
+      return NULL;
+    }
+  }
 
   ret = cdio_new (_data, &_funcs);
   if (ret == NULL) return NULL;
@@ -701,4 +868,3 @@ cdio_have_solaris (void)
   return false;
 #endif /* HAVE_SOLARIS_CDROM */
 }
-
