@@ -1,5 +1,5 @@
 /*
-    $Id: cdio.c,v 1.43 2004/03/21 03:43:06 rocky Exp $
+    $Id: cdio.c,v 1.56 2004/06/19 19:15:15 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
@@ -26,7 +26,9 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <errno.h>
 #include <string.h>
 
@@ -37,7 +39,7 @@
 #include <cdio/logging.h>
 #include "cdio_private.h"
 
-static const char _rcsid[] = "$Id: cdio.c,v 1.43 2004/03/21 03:43:06 rocky Exp $";
+static const char _rcsid[] = "$Id: cdio.c,v 1.56 2004/06/19 19:15:15 rocky Exp $";
 
 
 const char *track_format2str[6] = 
@@ -77,6 +79,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    NULL,
    NULL,
    NULL,
+   NULL,
    NULL
   },
 
@@ -86,6 +89,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    "BSDI ATAPI and SCSI driver",
    &cdio_have_bsdi,
    &cdio_open_bsdi,
+   &cdio_open_am_bsdi,
    &cdio_get_default_device_bsdi,
    &cdio_is_device_generic,
    &cdio_get_devices_bsdi
@@ -97,6 +101,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    "FreeBSD driver",
    &cdio_have_freebsd,
    &cdio_open_freebsd,
+   &cdio_open_am_freebsd,
    &cdio_get_default_device_freebsd,
    &cdio_is_device_generic,
    NULL
@@ -108,6 +113,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    "GNU/Linux ioctl and MMC driver",
    &cdio_have_linux,
    &cdio_open_linux,
+   &cdio_open_am_linux,
    &cdio_get_default_device_linux,
    &cdio_is_device_generic,
    &cdio_get_devices_linux
@@ -119,6 +125,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    "Solaris ATAPI and SCSI driver",
    &cdio_have_solaris,
    &cdio_open_solaris,
+   &cdio_open_am_solaris,
    &cdio_get_default_device_solaris,
    &cdio_is_device_generic,
    &cdio_get_devices_solaris
@@ -130,6 +137,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    "Apple Darwin OS X driver",
    &cdio_have_osx,
    &cdio_open_osx,
+   &cdio_open_am_osx,
    &cdio_get_default_device_osx,
    &cdio_is_device_generic,
    &cdio_get_devices_osx
@@ -138,20 +146,34 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
   {DRIVER_WIN32, 
    CDIO_SRC_IS_DEVICE_MASK|CDIO_SRC_IS_NATIVE_MASK|CDIO_SRC_IS_SCSI_MASK,
    "WIN32",
-   "Windows 32-bit ASPI and winNT/2K/XP ioctl driver",
+   "MS Windows ASPI and ioctl driver",
    &cdio_have_win32,
    &cdio_open_win32,
+   &cdio_open_am_win32,
    &cdio_get_default_device_win32,
    &cdio_is_device_win32,
    &cdio_get_devices_win32
   },
 
+  {DRIVER_CDRDAO,
+   CDIO_SRC_IS_DISK_IMAGE_MASK,
+   "CDRDAO",
+   "cdrdao (TOC) disk image driver",
+   &cdio_have_cdrdao,
+   &cdio_open_cdrdao,
+   &cdio_open_am_cdrdao,
+   &cdio_get_default_device_cdrdao,
+   NULL,
+   &cdio_get_devices_cdrdao
+  },
+
   {DRIVER_BINCUE,
    CDIO_SRC_IS_DISK_IMAGE_MASK,
    "BIN/CUE",
    "bin/cuesheet disk image driver",
    &cdio_have_bincue,
    &cdio_open_bincue,
+   &cdio_open_am_bincue,
    &cdio_get_default_device_bincue,
    NULL,
    &cdio_get_devices_bincue
@@ -163,6 +185,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    "Nero NRG disk image driver",
    &cdio_have_nrg,
    &cdio_open_nrg,
+   &cdio_open_am_nrg,
    &cdio_get_default_device_nrg,
    NULL,
    &cdio_get_devices_nrg
@@ -171,13 +194,15 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
 };
 
 static CdIo *
-scan_for_driver(driver_id_t start, driver_id_t end, const char *source_name) 
+scan_for_driver(driver_id_t start, driver_id_t end, 
+                const char *psz_source, const char *access_mode) 
 {
   driver_id_t driver_id;
   
   for (driver_id=start; driver_id<=end; driver_id++) {
     if ((*CdIo_all_drivers[driver_id].have_driver)()) {
-      CdIo *ret=(*CdIo_all_drivers[driver_id].driver_open)(source_name);
+      CdIo *ret=
+        (*CdIo_all_drivers[driver_id].driver_open_am)(psz_source, access_mode);
       if (ret != NULL) {
         ret->driver_id = driver_id;
         return ret;
@@ -291,7 +316,7 @@ cdio_get_devices (driver_id_t driver_id)
     /* FIXME: spit out unknown to give image drivers as well.  */
   case DRIVER_UNKNOWN:
   case DRIVER_DEVICE:
-    cdio = scan_for_driver(DRIVER_UNKNOWN, CDIO_MAX_DRIVER, NULL);
+    cdio = scan_for_driver(DRIVER_UNKNOWN, CDIO_MAX_DRIVER, NULL, NULL);
     break;
   default:
     return (*CdIo_all_drivers[driver_id].get_devices)();
@@ -363,7 +388,7 @@ cdio_get_devices_with_cap (char* search_devices[],
               ? (got_fs & need_fs_ext)  != 0
               : (got_fs | ~need_fs_ext) == -1;
             if (doit) 
-              cdio_add_device_list(&drives_ret, *drives, &num_drives);
+              cdio_add_device_list(&drives_ret, *d, &num_drives);
           }
              
         cdio_destroy(cdio);
@@ -376,6 +401,48 @@ cdio_get_devices_with_cap (char* search_devices[],
   return drives_ret;
 }
 
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+cdio_drive_cap_t
+cdio_get_drive_cap (const CdIo *cdio)
+{
+  cdio_drive_cap_t i_drivetype = 
+    CDIO_DRIVE_CAP_UNKNOWN | CDIO_DRIVE_CAP_CD_AUDIO; /* A safe guess. */
+  
+  if (cdio && cdio->op.get_drive_cap) {
+    i_drivetype=cdio->op.get_drive_cap(cdio->env);
+  }
+  return i_drivetype;
+}
+
+
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+cdio_drive_cap_t
+cdio_get_drive_cap_dev (const char *device) 
+{
+  cdio_drive_cap_t i_drivetype = CDIO_DRIVE_CAP_UNKNOWN;
+  
+  CdIo *cdio=scan_for_driver(CDIO_MIN_DRIVER, CDIO_MAX_DRIVER, 
+                             device, NULL);
+  if (cdio) {
+    i_drivetype=cdio_get_drive_cap(cdio);
+    cdio_destroy(cdio);
+  }
+  return i_drivetype;
+}
+
+
 /*!
   Return a string containing the name of the driver in use.
   if CdIo is NULL (we haven't initialized a specific device driver), 
@@ -384,9 +451,22 @@ cdio_get_devices_with_cap (char* search_devices[],
 const char *
 cdio_get_driver_name (const CdIo *cdio) 
 {
+  if (NULL==cdio) return NULL;
   return CdIo_all_drivers[cdio->driver_id].name;
 }
 
+  /*!
+    Return the driver id. 
+    if CdIo is NULL (we haven't initialized a specific device driver), 
+    then return DRIVER_UNKNOWN.
+  */
+driver_id_t
+cdio_get_driver_id (const CdIo *cdio) 
+{
+  if (NULL==cdio) return DRIVER_UNKNOWN;
+  return cdio->driver_id;
+}
+
 
 /*!
   Return the number of of the first track. 
@@ -563,10 +643,10 @@ cdio_have_driver(driver_id_t driver_id)
 }
 
 bool
-cdio_is_device(const char *source_name, driver_id_t driver_id)
+cdio_is_device(const char *psz_source, driver_id_t driver_id)
 {
   if (CdIo_all_drivers[driver_id].is_device == NULL) return false;
-  return (*CdIo_all_drivers[driver_id].is_device)(source_name);
+  return (*CdIo_all_drivers[driver_id].is_device)(psz_source);
 }
 
 
@@ -655,23 +735,32 @@ cdio_read (const CdIo *cdio, void *buf, size_t size)
   return -1;
 }
 
+/*!
+  Reads an audio sector from cd device into data starting
+  from lsn. Returns 0 if no error. 
+*/
 int
 cdio_read_audio_sector (const CdIo *cdio, void *buf, lsn_t lsn) 
 {
-  cdio_assert (cdio != NULL);
-  cdio_assert (buf != NULL);
+
+  if (NULL == cdio || NULL == buf || CDIO_INVALID_LSN == lsn )
+    return 0;
 
   if  (cdio->op.read_audio_sectors != NULL)
     return cdio->op.read_audio_sectors (cdio->env, buf, lsn, 1);
   return -1;
 }
 
+/*!
+  Reads audio sectors from cd device into data starting
+  from lsn. Returns 0 if no error. 
+*/
 int
 cdio_read_audio_sectors (const CdIo *cdio, void *buf, lsn_t lsn,
                          unsigned int nblocks) 
 {
-  cdio_assert (cdio != NULL);
-  cdio_assert (buf != NULL);
+  if ( NULL == cdio || NULL == buf || CDIO_INVALID_LSN == lsn )
+    return 0;
 
   if  (cdio->op.read_audio_sectors != NULL)
     return cdio->op.read_audio_sectors (cdio->env, buf, lsn, nblocks);
@@ -692,8 +781,8 @@ cdio_read_mode1_sector (const CdIo *cdio, void *data, lsn_t lsn, bool b_form2)
   uint32_t size = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE ;
   char buf[M2RAW_SECTOR_SIZE] = { 0, };
   
-  cdio_assert (cdio != NULL);
-  cdio_assert (data != NULL);
+  if (NULL == cdio || NULL == data || CDIO_INVALID_LSN == lsn )
+    return 0;
 
   if (cdio->op.read_mode1_sector && cdio->op.read_mode1_sector) {
     return cdio->op.read_mode1_sector(cdio->env, data, lsn, b_form2);
@@ -714,8 +803,10 @@ int
 cdio_read_mode1_sectors (const CdIo *cdio, void *buf, lsn_t lsn, 
                          bool b_form2,  unsigned int num_sectors)
 {
-  cdio_assert (cdio != NULL);
-  cdio_assert (buf != NULL);
+
+  if (NULL == cdio || NULL == buf || CDIO_INVALID_LSN == lsn )
+    return 0;
+
   cdio_assert (cdio->op.read_mode1_sectors != NULL);
   
   return cdio->op.read_mode1_sectors (cdio->env, buf, lsn, b_form2, 
@@ -730,8 +821,9 @@ int
 cdio_read_mode2_sector (const CdIo *cdio, void *buf, lsn_t lsn, 
                         bool b_form2)
 {
-  cdio_assert (cdio != NULL);
-  cdio_assert (buf != NULL);
+  if (NULL == cdio || NULL == buf || CDIO_INVALID_LSN == lsn )
+    return 0;
+
   cdio_assert (cdio->op.read_mode2_sector != NULL 
 	      || cdio->op.read_mode2_sectors != NULL);
 
@@ -748,8 +840,10 @@ int
 cdio_read_mode2_sectors (const CdIo *cdio, void *buf, lsn_t lsn, 
                          bool b_form2, unsigned int num_sectors)
 {
-  cdio_assert (cdio != NULL);
-  cdio_assert (buf != NULL);
+
+  if (NULL == cdio || NULL == buf || CDIO_INVALID_LSN == lsn )
+    return 0;
+
   cdio_assert (cdio->op.read_mode2_sectors != NULL);
   
   return cdio->op.read_mode2_sectors (cdio->env, buf, lsn,
@@ -787,58 +881,42 @@ cdio_set_arg (CdIo *cdio, const char key[], const char value[])
 CdIo *
 cdio_open (const char *orig_source_name, driver_id_t driver_id)
 {
-  char *source_name;
+  return cdio_open_am(orig_source_name, driver_id, NULL);
+}
+
+/*! Sets up to read from place specified by source_name and
+  driver_id. This should be called before using any other routine,
+  except cdio_init. This will call cdio_init, if that hasn't been
+  done previously.
+  
+  NULL is returned on error.
+*/
+CdIo *
+cdio_open_am (const char *psz_orig_source, driver_id_t driver_id,
+              const char *psz_access_mode)
+{
+  char *psz_source;
   
   if (CdIo_last_driver == -1) cdio_init();
 
-  if (NULL == orig_source_name || strlen(orig_source_name)==0) 
-    source_name = cdio_get_default_device(NULL);
+  if (NULL == psz_orig_source || strlen(psz_orig_source)==0) 
+    psz_source = cdio_get_default_device(NULL);
   else 
-    source_name = strdup(orig_source_name);
+    psz_source = strdup(psz_orig_source);
   
- retry:
   switch (driver_id) {
   case DRIVER_UNKNOWN: 
     {
       CdIo *cdio=scan_for_driver(CDIO_MIN_DRIVER, CDIO_MAX_DRIVER, 
-                                 source_name);
-      if (cdio != NULL && cdio_is_device(source_name, cdio->driver_id)) {
-        driver_id = cdio->driver_id;
-      } else {
-        struct stat buf;
-        if (0 != stat(source_name, &buf)) {
-          return NULL;
-        }
-        if (S_ISREG(buf.st_mode)) {
-        /* FIXME: check to see if is a text file. If so, then 
-           set SOURCE_CUE. */
-          int i=strlen(source_name)-strlen("bin");
-          if (i > 0
-              && ( (source_name)[i]   =='n' || (source_name)[i]   =='N' )
-              && ( (source_name)[i+1] =='r' || (source_name)[i+1] =='R' )
-              && ( (source_name)[i+2] =='g' || (source_name)[i+2] =='G' ) )
-            driver_id = DRIVER_NRG;
-          else if (i > 0
-                   && ( (source_name)[i]   =='c' || (source_name)[i]   =='C')
-                   && ( (source_name)[i+1] =='u' || (source_name)[i+1] =='U')
-                   && ( (source_name)[i+2] =='e' || (source_name)[i+2] =='E') )
-            driver_id = DRIVER_BINCUE;
-          else
-            driver_id = DRIVER_BINCUE;
-        } else {
-          cdio_destroy(cdio);
-          return NULL;
-        }
-        
-      }
-      cdio_destroy(cdio);
-      goto retry;
+                                 psz_source, psz_access_mode);
+      free(psz_source);
+      return cdio;
     }
   case DRIVER_DEVICE: 
     {  
       /* Scan for a driver. */
-      CdIo *ret = cdio_open_cd(source_name);
-      free(source_name);
+      CdIo *ret = cdio_open_am_cd(psz_source, psz_access_mode);
+      free(psz_source);
       return ret;
     }
     break;
@@ -850,19 +928,42 @@ cdio_open (const char *orig_source_name, driver_id_t driver_id)
   case DRIVER_OSX:
   case DRIVER_NRG:
   case DRIVER_BINCUE:
+  case DRIVER_CDRDAO:
     if ((*CdIo_all_drivers[driver_id].have_driver)()) {
-      CdIo *ret = (*CdIo_all_drivers[driver_id].driver_open)(source_name);
+      CdIo *ret = 
+        (*CdIo_all_drivers[driver_id].driver_open_am)(psz_source, 
+                                                      psz_access_mode);
       if (ret) ret->driver_id = driver_id;
-      free(source_name);
+      free(psz_source);
       return ret;
     }
   }
 
-  free(source_name);
+  free(psz_source);
   return NULL;
 }
 
 
+/*! 
+  Set up CD-ROM for reading. The device_name is
+  the some sort of device name.
+  
+  @return the cdio object for subsequent operations. 
+  NULL on error or there is no driver for a some sort of hardware CD-ROM.
+*/
+CdIo *
+cdio_open_cd (const char *psz_source)
+{
+  return cdio_open_am_cd(psz_source, NULL);
+}
+
+/*! 
+  Set up CD-ROM for reading. The device_name is
+  the some sort of device name.
+  
+  @return the cdio object for subsequent operations. 
+  NULL on error or there is no driver for a some sort of hardware CD-ROM.
+*/
 /* In the future we'll have more complicated code to allow selection
    of an I/O routine as well as code to find an appropriate default
    routine among the "registered" routines. Possibly classes too
@@ -872,13 +973,13 @@ cdio_open (const char *orig_source_name, driver_id_t driver_id)
    For now though, we'll start more simply...
 */
 CdIo *
-cdio_open_cd (const char *source_name)
+cdio_open_am_cd (const char *psz_source, const char *psz_access_mode)
 {
   if (CdIo_last_driver == -1) cdio_init();
 
   /* Scan for a driver. */
   return scan_for_driver(CDIO_MIN_DEVICE_DRIVER, CDIO_MAX_DEVICE_DRIVER, 
-                         source_name);
+                         psz_source, psz_access_mode);
 }
 
 
