@@ -1,5 +1,5 @@
 /*
-    $Id: cdio_private.h,v 1.19 2004/03/05 04:23:52 rocky Exp $
+    $Id: cdio_private.h,v 1.23 2004/05/27 01:11:50 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
 
@@ -69,11 +69,18 @@ extern "C" {
     */
     char * (*get_default_device)(void);
     
+    /*!
+      Return the what kind of device we've got.
+      
+      See cd_types.h for a list of bitmasks for the drive type;
+    */
+    cdio_drive_cap_t (*get_drive_cap) (const void *env);
+
     /*!  
       Return the media catalog number MCN from the CD or NULL if
       there is none or we don't have the ability to get it.
     */
-    char * (*get_mcn) (void *env);
+    char * (*get_mcn) (const void *env);
 
     /*!
       Return the number of of the first track. 
@@ -231,9 +238,11 @@ extern "C" {
     const char  *name;
     const char  *describe;
     bool (*have_driver) (void); 
-    CdIo *(*driver_open) (const char *source_name); 
+    CdIo *(*driver_open) (const char *psz_source_name); 
+    CdIo *(*driver_open_am) (const char *psz_source_name, 
+			     const char *psz_access_mode); 
     char *(*get_default_device) (void); 
-    bool (*is_device) (const char *source_name);
+    bool (*is_device) (const char *psz_source_name);
     char **(*get_devices) (void);
   } CdIo_driver_t;
 
