@@ -1,6 +1,7 @@
 /*
-    $Id: sector.c,v 1.4 2003/09/11 02:50:06 rocky Exp $
+    $Id: sector.c,v 1.11 2004/06/12 17:32:00 rocky Exp $
 
+    Copyright (C) 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -24,6 +25,7 @@
 
 #include <cdio/sector.h>
 #include <cdio/util.h>
+#include <cdio/logging.h>
 #include "cdio_assert.h"
 
 #include <stdio.h>
@@ -32,52 +34,80 @@
 #endif
 
 
-static const char _rcsid[] = "$Id: sector.c,v 1.4 2003/09/11 02:50:06 rocky Exp $";
+static const char _rcsid[] = "$Id: sector.c,v 1.11 2004/06/12 17:32:00 rocky Exp $";
 
 lba_t
 cdio_lba_to_lsn (lba_t lba)
 {
+  if (CDIO_INVALID_LBA     == lba) return CDIO_INVALID_LSN;
   return lba - CDIO_PREGAP_SECTORS; 
 }
 
+/* 
+   The below is adapted from cdparanoia code which claims it is
+   straight from the MMC3 spec.
+*/
+
 void
-cdio_lba_to_msf (lba_t lba, msf_t *msf)
+cdio_lsn_to_msf (lsn_t lsn, msf_t *msf)
 {
+  int m, s, f;
+  
   cdio_assert (msf != 0);
 
-  msf->m = to_bcd8 (lba / (60 * 75));
-  msf->s = to_bcd8 ((lba / 75) % 60);
-  msf->f = to_bcd8 (lba % 75);
+  if ( lsn >= -CDIO_PREGAP_SECTORS ){
+    m    = (lsn + CDIO_PREGAP_SECTORS) / CDIO_CD_FRAMES_PER_MIN;
+    lsn -= m * CDIO_CD_FRAMES_PER_MIN;
+    s    = (lsn + CDIO_PREGAP_SECTORS) / CDIO_CD_FRAMES_PER_SEC;
+    lsn -= s * CDIO_CD_FRAMES_PER_SEC;
+    f    = lsn + CDIO_PREGAP_SECTORS;
+  } else {
+    m    = (lsn + CDIO_CD_MAX_LSN)     / CDIO_CD_FRAMES_PER_MIN;
+    lsn -= m * (CDIO_CD_FRAMES_PER_MIN);
+    s    = (lsn+CDIO_CD_MAX_LSN)       / CDIO_CD_FRAMES_PER_SEC;
+    lsn -= s * CDIO_CD_FRAMES_PER_SEC;
+    f    = lsn + CDIO_CD_MAX_LSN;
+  }
+
+  if (m > 99) {
+    cdio_warn ("number of minutes (%d) truncated to 99.", m);
+    m = 99;
+  }
+  
+  msf->m = to_bcd8 (m);
+  msf->s = to_bcd8 (s);
+  msf->f = to_bcd8 (f);
 }
 
 /* warning, returns new allocated string */
 char *
 cdio_lba_to_msf_str (lba_t lba)
 {
-  char buf[16];
-  msf_t _msf = { .m = 0, .s = 0, .f = 0 };
 
-  cdio_lba_to_msf (lba, &_msf);
-
-  snprintf (buf, sizeof (buf), "%.2x:%.2x.%.2x", _msf.m, _msf.s, _msf.f);
-
-  return strdup (buf);
+  if (CDIO_INVALID_LBA == lba) {
+    return strdup("*INVALID");
+  } else {
+    msf_t msf = { .m = 0, .s = 0, .f = 0 };
+    cdio_lba_to_msf (lba, &msf);
+    return cdio_msf_to_str(&msf);
+  }
 }
 
 lba_t
 cdio_lsn_to_lba (lsn_t lsn)
 {
+  if (CDIO_INVALID_LSN  == lsn) return CDIO_INVALID_LBA;
   return lsn + CDIO_PREGAP_SECTORS; 
 }
 
 void
-cdio_lsn_to_msf (lsn_t lsn, msf_t *msf)
+cdio_lba_to_msf (lba_t lba, msf_t *msf)
 {
   cdio_assert (msf != 0);
-  cdio_lba_to_msf(cdio_lsn_to_lba(lsn), msf);
+  cdio_lsn_to_msf(cdio_lba_to_lsn(lba), msf);
 }
 
-uint32_t
+lba_t
 cdio_msf_to_lba (const msf_t *msf)
 {
   uint32_t lba = 0;
@@ -85,22 +115,32 @@ cdio_msf_to_lba (const msf_t *msf)
   cdio_assert (msf != 0);
 
   lba = from_bcd8 (msf->m);
-  lba *= 60;
+  lba *= CDIO_CD_SECS_PER_MIN;
 
   lba += from_bcd8 (msf->s);
-  lba *= 75;
+  lba *= CDIO_CD_FRAMES_PER_SEC;
   
   lba += from_bcd8 (msf->f);
 
   return lba;
 }
 
-uint32_t
+lba_t
 cdio_msf_to_lsn (const msf_t *msf)
 {
   return cdio_lba_to_lsn(cdio_msf_to_lba (msf));
 }
 
+/* warning, returns new allocated string */
+char *
+cdio_msf_to_str (const msf_t *msf)
+{
+  char buf[16];
+  
+  snprintf (buf, sizeof (buf), "%2.2x:%2.2x:%2.2x", msf->m, msf->s, msf->f);
+  return strdup (buf);
+}
+
 
 /* 
  * Local variables:
