@@ -1,5 +1,5 @@
 /*
-    $Id: cd-info.c,v 1.50 2004/03/20 22:46:57 rocky Exp $
+    $Id: cd-info.c,v 1.70 2004/06/23 09:28:02 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 1996, 1997, 1998  Gerd Knorr <kraxel@bytesex.org>
@@ -105,6 +105,7 @@ enum {
   OP_SOURCE_AUTO,
   OP_SOURCE_BIN,
   OP_SOURCE_CUE,
+  OP_SOURCE_CDRDAO,
   OP_SOURCE_NRG ,
   OP_SOURCE_DEVICE,
   
@@ -121,7 +122,8 @@ static bool
 parse_options (int argc, const char *argv[])
 {
   int opt;
-
+  char *psz_my_source;
+  
   struct poptOption optionsTable[] = {
     {"access-mode",       'a', POPT_ARG_STRING, &opts.access_mode, 0,
      "Set CD access methed"},
@@ -136,8 +138,8 @@ parse_options (int argc, const char *argv[])
      "Don't filesystem analysis"},
     
 #ifdef HAVE_CDDB
-  {"no-cddb",     'a', POPT_ARG_NONE, &opts.no_cddb, 0,
-   "Don't look up audio CDDB information or print that"},
+    {"no-cddb",     '\0', POPT_ARG_NONE, &opts.no_cddb, 0,
+     "Don't look up audio CDDB information or print that"},
     
     {"cddb-port",   'P', POPT_ARG_INT, &opts.cddb_port, 8880,
      "CDDB port number to use (default 8880)"},
@@ -171,13 +173,13 @@ parse_options (int argc, const char *argv[])
     {"no-ioctl",  'I', POPT_ARG_NONE,  &opts.no_ioctl, 0,
      "Don't show ioctl() information"},
     
-    {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_BIN, "set \"bin\" CD-ROM disk image file as source", "FILE"},
     
-    {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_CUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
     
-    {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_AUTO,
      "set source and determine if \"bin\" image or device", "FILE"},
     
@@ -194,9 +196,12 @@ parse_options (int argc, const char *argv[])
     {"no-header", '\0', POPT_ARG_NONE, &opts.no_header, 
      0, "Don't display header and copyright (for regression testing)"},
     
-    {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_NRG, "set Nero CD-ROM disk image file as source", "FILE"},
     
+    {"toc-file", 't', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
+     OP_SOURCE_CDRDAO, "set cdrdao CD-ROM disk image file as source", "FILE"},
+    
     {"quiet",       'q', POPT_ARG_NONE, &opts.silent, 0,
      "Don't produce warning output" },
     
@@ -215,6 +220,7 @@ parse_options (int argc, const char *argv[])
     case OP_SOURCE_AUTO:
     case OP_SOURCE_BIN: 
     case OP_SOURCE_CUE: 
+    case OP_SOURCE_CDRDAO: 
     case OP_SOURCE_NRG: 
     case OP_SOURCE_DEVICE: 
       if (opts.source_image != IMAGE_UNKNOWN) {
@@ -225,6 +231,14 @@ parse_options (int argc, const char *argv[])
 		program_name);
 	break;
       } 
+
+      /* For all input sources which are not a DEVICE, we need to make
+	 a copy of the string; for a DEVICE the fill-out routine makes
+	 the copy.
+       */
+      if (OP_SOURCE_DEVICE != opt) 
+	source_name = strdup(psz_my_source);
+      
       switch (opt) {
       case OP_SOURCE_BIN: 
 	opts.source_image  = IMAGE_BIN;
@@ -232,6 +246,9 @@ parse_options (int argc, const char *argv[])
       case OP_SOURCE_CUE: 
 	opts.source_image  = IMAGE_CUE;
 	break;
+      case OP_SOURCE_CDRDAO: 
+	opts.source_image  = IMAGE_CDRDAO;
+	break;
       case OP_SOURCE_NRG: 
 	opts.source_image  = IMAGE_NRG;
 	break;
@@ -255,8 +272,9 @@ parse_options (int argc, const char *argv[])
     if ( remaining_arg != NULL) {
       if (opts.source_image != IMAGE_UNKNOWN) {
 	fprintf (stderr, 
-		 "%s: Source specified in option %s and as %s\n", 
-		 program_name, source_name, remaining_arg);
+		 "%s: Source '%s' given as an argument of an option and as "
+		 "unnamed option '%s'\n", 
+		 program_name, psz_my_source, remaining_arg);
 	poptFreeContext(optCon);
 	free(program_name);
 	exit (EXIT_FAILURE);
@@ -270,7 +288,7 @@ parse_options (int argc, const char *argv[])
       if ( (poptGetArgs(optCon)) != NULL) {
 	fprintf (stderr, 
 		 "%s: Source specified in previously %s and %s\n", 
-		 program_name, source_name, remaining_arg);
+		 program_name, psz_my_source, remaining_arg);
 	poptFreeContext(optCon);
 	free(program_name);
 	exit (EXIT_FAILURE);
@@ -426,7 +444,7 @@ print_cddb_info(CdIo *cdio, track_t num_tracks, track_t first_track_num) {
   matches = cddb_query(conn, disc);
 
   if (-1 == matches) 
-    printf("%s: %s\n", program_name, cddb_error_str(errno));
+    printf("%s: %s\n", program_name, cddb_error_str(cddb_errno(conn)));
   else {
     printf("%s: Found %d matches in CDDB\n", program_name, matches);
     for (i=1; i<=matches; i++) {
@@ -444,10 +462,10 @@ print_cddb_info(CdIo *cdio, track_t num_tracks, track_t first_track_num) {
 
 #ifdef HAVE_VCDINFO
 static void 
-print_vcd_info(void) {
+print_vcd_info(driver_id_t driver) {
   vcdinfo_open_return_t open_rc;
   vcdinfo_obj_t *obj;
-  open_rc = vcdinfo_open(&obj, &source_name, DRIVER_UNKNOWN, NULL);
+  open_rc = vcdinfo_open(&obj, &source_name, driver, NULL);
   switch (open_rc) {
   case VCDINFO_OPEN_VCD: 
     if (vcdinfo_get_format_version (obj) == VCD_TYPE_INVALID) {
@@ -455,17 +473,11 @@ print_vcd_info(void) {
       vcdinfo_close(obj);
       return;
     }
-    fprintf (stdout, "format: %s\n", vcdinfo_get_format_version_str(obj));
-    fprintf (stdout, "album id: `%.16s'\n",  vcdinfo_get_album_id(obj));
-    fprintf (stdout, "volume count: %d\n",   vcdinfo_get_volume_count(obj));
+    fprintf (stdout, "Format      : %s\n", 
+	     vcdinfo_get_format_version_str(obj));
+    fprintf (stdout, "Album       : `%.16s'\n",  vcdinfo_get_album_id(obj));
+    fprintf (stdout, "Volume count: %d\n",   vcdinfo_get_volume_count(obj));
     fprintf (stdout, "volume number: %d\n",  vcdinfo_get_volume_num(obj));
-    fprintf (stdout, "system id: `%s'\n",    vcdinfo_get_system_id(obj));
-    fprintf (stdout, "volume id: `%s'\n",    vcdinfo_get_volume_id(obj));
-    fprintf (stdout, "volumeset id: `%s'\n", vcdinfo_get_volumeset_id(obj));
-    fprintf (stdout, "publisher id: `%s'\n", vcdinfo_get_publisher_id(obj));
-    fprintf (stdout, "preparer id: `%s'\n",  vcdinfo_get_preparer_id(obj));
-    fprintf (stdout, "application id: `%s'\n", 
-	     vcdinfo_get_application_id(obj));
 
     break;
   case VCDINFO_OPEN_ERROR:
@@ -483,14 +495,15 @@ print_vcd_info(void) {
 #endif 
 
 static void
-print_iso9660_recurse (CdIo *cdio, const char pathname[], cdio_fs_anal_t fs, 
-		       bool is_mode2)
+print_iso9660_recurse (const CdIo *cdio, const char pathname[], 
+		       cdio_fs_anal_t fs, 
+		       bool b_mode2)
 {
   CdioList *entlist;
   CdioList *dirlist =  _cdio_list_new ();
   CdioListNode *entnode;
 
-  entlist = iso9660_fs_readdir (cdio, pathname, is_mode2);
+  entlist = iso9660_fs_readdir (cdio, pathname, b_mode2);
     
   printf ("%s:\n", pathname);
 
@@ -539,7 +552,7 @@ print_iso9660_recurse (CdIo *cdio, const char pathname[], cdio_fs_anal_t fs,
 	  printf ("%9u", (unsigned int) statbuf->size);
 	}
       }
-      strftime(date_str, DATESTR_SIZE, "%b %e %Y %H:%M ", &statbuf->tm);
+      strftime(date_str, DATESTR_SIZE, "%b %d %Y %H:%M ", &statbuf->tm);
       printf (" %s %s\n", date_str, translated_name);
     }
 
@@ -553,38 +566,50 @@ print_iso9660_recurse (CdIo *cdio, const char pathname[], cdio_fs_anal_t fs,
     {
       char *_fullname = _cdio_list_node_data (entnode);
 
-      print_iso9660_recurse (cdio, _fullname, fs, is_mode2);
+      print_iso9660_recurse (cdio, _fullname, fs, b_mode2);
     }
 
   _cdio_list_free (dirlist, true);
 }
 
-static void
-print_iso9660_fs (CdIo *cdio, cdio_fs_anal_t fs, track_format_t track_format)
-{
-  iso9660_pvd_t pvd;
-  bool is_mode2 = false;
-
-  if (fs & CDIO_FS_ANAL_XA) track_format = TRACK_FORMAT_XA;
-
+static bool
+read_iso9660_pvd(const CdIo *p_cdio, track_format_t track_format, /*out*/
+		 iso9660_pvd_t *p_pvd) 		 {
+  
   switch (track_format) {
   case TRACK_FORMAT_CDI:
   case TRACK_FORMAT_XA:
-    if (cdio_read_mode2_sector (cdio, &pvd, ISO_PVD_SECTOR, false))
-      return;
-    is_mode2 = true;
+    if (0 != cdio_read_mode2_sector (p_cdio, p_pvd, ISO_PVD_SECTOR, false))
+      return false;
     break;
   case TRACK_FORMAT_DATA:
-    if (cdio_read_mode1_sector (cdio, &pvd, ISO_PVD_SECTOR, false))
-      return;
-    is_mode2 = false;
+    if (0 != cdio_read_mode1_sector (p_cdio, p_pvd, ISO_PVD_SECTOR, false))
+      return false;
     break;
   case TRACK_FORMAT_AUDIO: 
   case TRACK_FORMAT_PSX: 
   case TRACK_FORMAT_ERROR: 
   default:
-    return;
+    return false;
   }
+  return true;
+}
+		 
+
+static void
+print_iso9660_fs (const CdIo *p_cdio, cdio_fs_anal_t fs, 
+		  track_format_t track_format)
+{
+  iso9660_pvd_t pvd;
+  bool b_mode2 = false;
+
+  if (fs & CDIO_FS_ANAL_XA) track_format = TRACK_FORMAT_XA;
+
+  if ( !read_iso9660_pvd(p_cdio, track_format, &pvd) ) 
+    return;
+  
+  b_mode2 = ( TRACK_FORMAT_CDI == track_format 
+	       || TRACK_FORMAT_XA == track_format );
   
   {
     const lsn_t extent = iso9660_get_root_lsn(&pvd);
@@ -592,7 +617,7 @@ print_iso9660_fs (CdIo *cdio, cdio_fs_anal_t fs, track_format_t track_format)
     printf ("ISO9660 filesystem\n");
     printf (" root dir in PVD set to lsn %lu\n\n", (long unsigned) extent);
     
-    print_iso9660_recurse (cdio, "/", fs, is_mode2);
+    print_iso9660_recurse (p_cdio, "/", fs, b_mode2);
   }
 }
 
@@ -600,7 +625,7 @@ static void
 print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis, 
 	       cdio_fs_anal_t fs, int first_data, unsigned int num_audio, 
 	       track_t num_tracks, track_t first_track_num, 
-	       track_format_t track_format, CdIo *cdio)
+	       track_format_t track_format, CdIo *p_cdio)
 {
   int need_lf;
   
@@ -608,9 +633,9 @@ print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis,
   case CDIO_FS_AUDIO:
     if (num_audio > 0) {
       printf("Audio CD, CDDB disc ID is %08lx\n", 
-	     cddb_discid(cdio, num_tracks));
+	     cddb_discid(p_cdio, num_tracks));
 #ifdef HAVE_CDDB
-      if (!opts.no_cddb) print_cddb_info(cdio, num_tracks, first_track_num);
+      if (!opts.no_cddb) print_cddb_info(p_cdio, num_tracks, first_track_num);
 #endif      
     }
     break;
@@ -642,25 +667,59 @@ print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis,
     printf("CD-ROM with Unix UFS\n");
     break;
   case CDIO_FS_EXT2:
-    printf("CD-ROM with Linux second extended filesystem\n");
+    printf("CD-ROM with GNU/Linux EXT2 (native) filesystem\n");
 	  break;
   case CDIO_FS_3DO:
     printf("CD-ROM with Panasonic 3DO filesystem\n");
     break;
+  case CDIO_FS_UDFX:
+    printf("CD-ROM with UDFX filesystem\n");
+    break;
   case CDIO_FS_UNKNOWN:
     printf("CD-ROM with unknown filesystem\n");
     break;
+  case CDIO_FS_XISO:
+    printf("CD-ROM with Microsoft X-BOX XISO filesystem\n");
+    break;
   }
   switch(CDIO_FSTYPE(fs)) {
   case CDIO_FS_ISO_9660:
   case CDIO_FS_ISO_9660_INTERACTIVE:
   case CDIO_FS_ISO_HFS:
+  case CDIO_FS_ISO_UDF:
     printf("ISO 9660: %i blocks, label `%.32s'\n",
 	   cdio_iso_analysis.isofs_size, cdio_iso_analysis.iso_label);
+
+    {
+      iso9660_pvd_t pvd;
+
+      if ( read_iso9660_pvd(p_cdio, track_format, &pvd) ) {
+	fprintf(stdout, "Application: %s\n", 
+		iso9660_get_application_id(&pvd));
+	fprintf(stdout, "Preparer   : %s\n", iso9660_get_preparer_id(&pvd));
+	fprintf(stdout, "Publisher  : %s\n", iso9660_get_publisher_id(&pvd));
+	fprintf(stdout, "System     : %s\n", iso9660_get_system_id(&pvd));
+	fprintf(stdout, "Volume     : %s\n", iso9660_get_volume_id(&pvd));
+	fprintf(stdout, "Volume Set : %s\n", iso9660_get_volumeset_id(&pvd));
+      }
+      
+    }
+    
     if (opts.print_iso9660) 
-      print_iso9660_fs(cdio, fs, track_format);
+      print_iso9660_fs(p_cdio, fs, track_format);
+    
     break;
   }
+  
+  switch(CDIO_FSTYPE(fs)) {
+  case CDIO_FS_UDF:
+  case CDIO_FS_ISO_UDF:
+    fprintf(stdout, "UDF: version %x.%2.2x\n",
+	    cdio_iso_analysis.UDFVerMajor, cdio_iso_analysis.UDFVerMinor);
+    break;
+  default: ;
+  }
+
   need_lf = 0;
   if (first_data == 1 && num_audio > 0)
     need_lf += printf("mixed mode CD   ");
@@ -691,7 +750,7 @@ print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis,
   if (fs & (CDIO_FS_ANAL_VIDEOCD|CDIO_FS_ANAL_CVD|CDIO_FS_ANAL_SVCD)) 
     if (!opts.no_vcd) {
       printf("\n");
-      print_vcd_info();
+      print_vcd_info(cdio_get_driver_id(p_cdio));
     }
 #endif    
 
@@ -730,6 +789,7 @@ init(void)
   opts.no_ioctl      = 0;
   opts.no_analysis   = 0;
   opts.source_image  = IMAGE_UNKNOWN;
+  opts.access_mode   = NULL;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -777,44 +837,89 @@ main(int argc, const char *argv[])
   switch (opts.source_image) {
   case IMAGE_UNKNOWN:
   case IMAGE_AUTO:
-    cdio = cdio_open (source_name, DRIVER_UNKNOWN);
+    cdio = cdio_open_am (source_name, DRIVER_UNKNOWN, opts.access_mode);
     if (cdio==NULL) {
-      err_exit("%s: Error in automatically selecting driver with input\n", 
-	       program_name);
+      if (source_name) {
+	err_exit("%s: Error in automatically selecting driver for input %s.\n", 
+		 program_name, source_name);
+      } else {
+	err_exit("%s: Error in automatically selecting driver.\n", 
+		 program_name);
+      }
     } 
     break;
   case IMAGE_DEVICE:
-    cdio = cdio_open (source_name, DRIVER_DEVICE);
+    cdio = cdio_open_am (source_name, DRIVER_DEVICE, opts.access_mode);
     if (cdio==NULL) {
-      err_exit("%s: Error in automatically selecting device with input\n", 
+      if (source_name) {
+	err_exit("%s: Error in automatically selecting CD-image driver for input %s\n", 
+	       program_name, source_name);
+      } else {
+	err_exit("%s: Error in automatically selecting CD-image driver.\n", 
 	       program_name);
+      }
     } 
     break;
   case IMAGE_BIN:
-    cdio = cdio_open (source_name, DRIVER_BINCUE);
+    cdio = cdio_open_am (source_name, DRIVER_BINCUE, opts.access_mode);
     if (cdio==NULL) {
-      err_exit("%s: Error in opeing bin/cue\n", 
-	       program_name);
+      if (source_name) {
+	err_exit("%s: Error in opening bin/cue for input %s\n", 
+		 program_name, source_name);
+      } else {
+	err_exit("%s: Error in opening bin/cue for unspecified input.\n", 
+		 program_name);
+      }
     } 
     break;
   case IMAGE_CUE:
     cdio = cdio_open_cue(source_name);
     if (cdio==NULL) {
-      err_exit("%s: Error in opening cue/bin with input\n", 
-	       program_name);
+      if (source_name) {
+	err_exit("%s: Error in opening cue/bin with input %s\n", 
+		 program_name, source_name);
+      } else {
+	err_exit("%s: Error in opening cue/bin for unspeicified input.\n", 
+		 program_name);
+      }
     } 
     break;
   case IMAGE_NRG:
-    cdio = cdio_open (source_name, DRIVER_NRG);
+    cdio = cdio_open_am (source_name, DRIVER_NRG, opts.access_mode);
     if (cdio==NULL) {
-      err_exit("%s: Error in opening NRG with input\n", 
-	       program_name);
+      if (source_name) {
+	err_exit("%s: Error in opening NRG with input %s\n", 
+		 program_name, source_name);
+      } else {
+	err_exit("%s: Error in opening NRG for unspecified input.\n", 
+		 program_name);
+      }
     } 
     break;
+
+  case IMAGE_CDRDAO:
+    cdio = cdio_open_am (source_name, DRIVER_CDRDAO, opts.access_mode);
+    if (cdio==NULL) {
+      if (source_name) {
+	err_exit("%s: Error in opening TOC with input %s.\n", 
+		 program_name, source_name);
+      } else {
+	err_exit("%s: Error in opening TOC for unspecified input.\n", 
+		 program_name);
+      }
+    }
+    break;
   }
 
   if (cdio==NULL) {
-    err_exit("%s: Error in opening device driver\n", program_name);
+    if (source_name) {
+      err_exit("%s: Error in opening device driver for input %s.\n", 
+	       program_name, source_name);
+    } else {
+      err_exit("%s: Error in opening device driver for unspecified input.\n", 
+	       program_name);
+    }
+    
   } 
 
   if (source_name==NULL) {
@@ -825,10 +930,16 @@ main(int argc, const char *argv[])
   } 
 
   if (opts.silent == 0) {
-    printf("CD location   : %s\n", source_name);
-    printf("CD driver name: %s\n", cdio_get_driver_name(cdio));
+    printf("CD location   : %s\n",   source_name);
+    printf("CD driver name: %s\n",   cdio_get_driver_name(cdio));
+    printf("   access mode: %s\n\n", cdio_get_arg(cdio, "access-mode"));
+  } 
+
+  {
+    cdio_drive_cap_t i_drive_cap =  cdio_get_drive_cap(cdio);
+    print_drive_capabilities(i_drive_cap);
   }
-  
+
   if (opts.list_drives) {
     char ** device_list = cdio_get_devices(DRIVER_DEVICE);
     char ** d = device_list;
@@ -840,13 +951,9 @@ main(int argc, const char *argv[])
       }
     }
     cdio_free_device_list(device_list);
-    free(device_list);
+    if (device_list) free(device_list);
   }
 
-  if (opts.access_mode!=NULL) {
-    cdio_set_arg(cdio, "access-mode", opts.access_mode);
-  } 
-
   first_track_num = cdio_get_first_track_num(cdio);
   num_tracks      = cdio_get_num_tracks(cdio);
 
@@ -860,22 +967,20 @@ main(int argc, const char *argv[])
   /* Read and possibly print track information. */
   for (i = first_track_num; i <= CDIO_CDROM_LEADOUT_TRACK; i++) {
     msf_t msf;
-    
+    char *psz_msf;
+
     if (!cdio_get_track_msf(cdio, i, &msf)) {
       err_exit("cdio_track_msf for track %i failed, I give up.\n", i);
     }
 
+    psz_msf = cdio_msf_to_str(&msf);
     if (i == CDIO_CDROM_LEADOUT_TRACK) {
       if (!opts.no_tracks)
-	printf("%3d: %2.2x:%2.2x:%2.2x  %06lu  leadout\n",
-	       (int) i, 
-	       msf.m, msf.s, msf.f, 
+	printf("%3d: %8s  %06lu  leadout\n", (int) i, psz_msf,
 	       (long unsigned int) cdio_msf_to_lsn(&msf));
       break;
     } else if (!opts.no_tracks) {
-      printf("%3d: %2.2x:%2.2x:%2.2x  %06lu  %-5s %s\n",
-	     (int) i, 
-	     msf.m, msf.s, msf.f, 
+      printf("%3d: %8s  %06lu  %-5s %s\n", (int) i, psz_msf,
 	     (long unsigned int) cdio_msf_to_lsn(&msf),
 	     track_format2str[cdio_get_track_format(cdio, i)],
 	     cdio_get_track_green(cdio, i)? "true" : "false");
