@@ -1,7 +1,7 @@
 /*
-  $Id: cd-read.c,v 1.16 2004/02/07 02:40:20 rocky Exp $
+  $Id: cd-read.c,v 1.19 2004/05/31 14:52:04 rocky Exp $
 
-  Copyright (C) 2003 Rocky Bernstein <rocky@panix.com>
+  Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ enum {
   OP_SOURCE_BIN,
   OP_SOURCE_CUE,
   OP_SOURCE_NRG         = DRIVER_NRG,
+  OP_SOURCE_CDRDAO         = DRIVER_CDRDAO,
   OP_SOURCE_DEVICE      = DRIVER_DEVICE,
 
   /* These are the remaining configuration options */
@@ -91,6 +92,11 @@ struct arguments
   char          *access_mode; /* Access method driver should use for control */
   char          *output_file; /* file to output blocks if not NULL. */
   int            debug_level;
+  int            hexdump;     /* Show output as a hexdump */
+  int            nohexdump;   /* Don't output as a hexdump. I don't know
+				 how to get popt to combine these as 
+				 one variable.
+			       */
   read_mode_t    read_mode;
   int            version_only;
   int            no_header;
@@ -102,26 +108,27 @@ struct arguments
 } opts;
      
 static void
-hexdump (uint8_t * buffer, unsigned int len)
+hexdump (FILE *stream,  uint8_t * buffer, unsigned int len)
 {
   unsigned int i;
   for (i = 0; i < len; i++, buffer++)
     {
       if (i % 16 == 0)
-	printf ("0x%04x: ", i);
-      printf ("%02x", *buffer);
+	fprintf (stream, "0x%04x: ", i);
+      fprintf (stream, "%02x", *buffer);
       if (i % 2 == 1)
-	printf (" ");
+	fprintf (stream, " ");
       if (i % 16 == 15) {
 	uint8_t *p; 
-	printf ("  ");
+	fprintf (stream, "  ");
 	for (p=buffer-15; p <= buffer; p++) {
-	  printf("%c", isprint(*p) ?  *p : '.');
+	  fprintf(stream, "%c", isprint(*p) ?  *p : '.');
 	}
-	printf ("\n");
+	fprintf (stream, "\n");
       }
     }
-  printf ("\n");
+  fprintf (stream, "\n");
+  fflush (stream);
 }
 
 /* Comparison function called by bearch() to find sub-option record. */
@@ -172,6 +179,7 @@ parse_options (int argc, const char *argv[])
 {
 
   int opt;
+  char *psz_my_source;
   char *opt_arg;
 
   /* Command-line options */
@@ -190,6 +198,13 @@ parse_options (int argc, const char *argv[])
      POPT_ARG_INT, &opts.debug_level, 0,
      "Set debugging to LEVEL"},
     
+    {"hexdump",  'x', POPT_ARG_NONE, &opts.hexdump, 0,
+     "Show output as a hex dump. The default is a hex dump when "
+     "output goes to stdout and no hex dump when output is to a file."},
+
+    {"no-hexdump",  '\0', POPT_ARG_NONE, &opts.nohexdump, 0,
+     "Don't show output as a hex dump."},
+
     {"start",       's', 
      POPT_ARG_INT, &opts.start_lsn, 0,
      "Set LBA to start reading from"},
@@ -202,25 +217,28 @@ parse_options (int argc, const char *argv[])
      POPT_ARG_INT, &opts.num_sectors, 0,
      "Set number of sectors to read"},
     
-    {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_BIN, "set \"bin\" CD-ROM disk image file as source", "FILE"},
     
-    {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_CUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
     
-    {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_AUTO,
      "set source and determine if \"bin\" image or device", "FILE"},
     
-    {"cdrom-device", 'C', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"cdrom-device", 'C', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name,
      OP_SOURCE_DEVICE,
      "set CD-ROM device as source", "DEVICE"},
     
-    {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+    {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_NRG, "set Nero CD-ROM disk image file as source", "FILE"},
     
+    {"toc-file", 't', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
+     OP_SOURCE_CDRDAO, "set \"TOC\" CD-ROM disk image file as source", "FILE"},
+    
     {"output-file",     'o', POPT_ARG_STRING, &opts.output_file, 0,
-     "Output blocks to file rather than give a hexdump."},
+     "Output blocks to file rather than give a hexdump.", "FILE"},
     
     {"version", 'V', POPT_ARG_NONE, NULL, OP_VERSION,
      "display version and copyright information and exit"},
@@ -247,6 +265,14 @@ parse_options (int argc, const char *argv[])
 		  program_name);
 	  break;
 	} 
+
+	/* For all input sources which are not a DEVICE, we need to make
+	   a copy of the string; for a DEVICE the fill-out routine makes
+	   the copy.
+	*/
+	if (OP_SOURCE_DEVICE != opt) 
+	  source_name = strdup(psz_my_source);
+
 	switch (opt) {
 	case OP_SOURCE_BIN: 
 	  opts.source_image  = IMAGE_BIN;
@@ -335,6 +361,15 @@ parse_options (int argc, const char *argv[])
 
   /* Check consistency between start_lsn, end_lsn and num_sectors. */
 
+  if (opts.nohexdump && opts.hexdump != 2) {
+    fprintf(stderr, 
+	    "%s: don't give both --hexdump and --no-hexdump together\n",
+	    program_name);
+    exit(13);
+  }
+
+  if (opts.nohexdump) opts.hexdump = 0;
+  
   if (opts.start_lsn == CDIO_INVALID_LSN) {
     /* Maybe we derive the start from the end and num sectors. */
     if (opts.end_lsn == CDIO_INVALID_LSN) {
@@ -414,6 +449,7 @@ init(void)
   opts.num_sectors   = 0;
   opts.read_mode     = READ_MODE_UNINIT;
   opts.source_image  = IMAGE_UNKNOWN;
+  opts.hexdump       = 2;         /* Not set. */
 
   gl_default_cdio_log_handler = cdio_log_set_handler (log_handler);
 }
@@ -425,6 +461,7 @@ main(int argc, const char *argv[])
   unsigned int blocklen=CDIO_CD_FRAMESIZE_RAW;
   CdIo *cdio=NULL;
   int output_fd=-1;
+  FILE *output_stream;
   
   init();
 
@@ -472,6 +509,14 @@ main(int argc, const char *argv[])
 	       source_name);
     } 
     break;
+
+  case IMAGE_CDRDAO:
+    cdio = cdio_open (source_name, DRIVER_CDRDAO);
+    if (cdio==NULL) {
+      err_exit("Error in opening TOC file %s for input\n", 
+	       source_name);
+    } 
+    break;
   }
 
   if (opts.access_mode!=NULL) {
@@ -479,13 +524,23 @@ main(int argc, const char *argv[])
   } 
 
   if (opts.output_file!=NULL) {
+    
+    /* If hexdump not explicitly set, then don't produce hexdump 
+       when writing to a file.
+     */
+    if (opts.hexdump == 2) opts.hexdump = 0;
+
     output_fd = open(opts.output_file, O_WRONLY|O_CREAT|O_TRUNC, 0644);
     if (-1 == output_fd) {
       err_exit("Error opening output file %s: %s\n",
 	       opts.output_file, strerror(errno));
 
     }
-  } 
+  } else 
+    /* If we are writing to stdout, then the default is to produce 
+       a hexdump.
+     */
+    if (opts.hexdump == 2) opts.hexdump = 1;
 
 
   for ( ; opts.start_lsn <= opts.end_lsn; opts.start_lsn++ ) {
@@ -543,11 +598,21 @@ main(int argc, const char *argv[])
       break;
     }
 
-    if (opts.output_file) {
-      write(output_fd, buffer, blocklen);
+    if (!opts.output_file) {
+      output_stream = stdout;
     } else {
-      hexdump(buffer, blocklen);
+      output_stream = fdopen(output_fd, "w");
     }
+    
+    if (opts.hexdump)
+      hexdump(output_stream, buffer, blocklen);
+    else if (opts.output_file)
+      write(output_fd, buffer, blocklen);
+    else {
+      unsigned int i;
+      for (i=0; i<blocklen; i++) printf("%c", buffer[i]);
+    }
+    
   }
 
   if (opts.output_file) close(output_fd);
