@@ -1,7 +1,7 @@
 /*
-  $Id: util.h,v 1.2 2004/02/07 02:40:20 rocky Exp $
+  $Id: util.h,v 1.4 2004/05/04 02:06:48 rocky Exp $
 
-  Copyright (C) 2003 Rocky Bernstein <rocky@panix.com>
+  Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -85,6 +85,7 @@ typedef enum
   IMAGE_BIN,
   IMAGE_CUE,
   IMAGE_NRG,
+  IMAGE_CDRDAO,
   IMAGE_UNKNOWN
 } source_image_t;
 
@@ -99,4 +100,7 @@ void print_version (char *program_name, const char *version,
 
 char *fillout_device_name(const char *device_name);
 
+/* Prints out drive capabilities */
+void print_drive_capabilities(cdio_drive_cap_t i_drive_cap);
+
 #endif /* UTIL_H */
