@@ -1,5 +1,5 @@
 #!/bin/sh
-#$Id: check_nrg.sh.in,v 1.9 2003/10/06 04:04:05 rocky Exp $
+#$Id: check_nrg.sh.in,v 1.10 2004/05/31 13:32:19 rocky Exp $
 
 if test -n "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  " ; then
   vcd_opt='--no-vcd'
@@ -16,10 +16,10 @@ if test ! -x ../src/cd-info ; then
 fi
 
 BASE=`basename $0 .sh`
-fname=videocd
+test_name=videocd
 
-test_cdinfo "--quiet --nrg-file ${srcdir}/${fname}.nrg $vcd_opt --iso9660 " \
-  ${fname}.dump ${srcdir}/${fname}.right
+test_cdinfo "--quiet --nrg-file ${srcdir}/${test_name}.nrg $vcd_opt --iso9660 " \
+  ${test_name}.dump ${srcdir}/${test_name}.right
 RC=$?
 check_result $RC 'cd-info NRG test 1'
 
@@ -36,12 +36,25 @@ else
   exit 0
 fi
 
-nrg_file=${srcdir}/svcdgs.nrg
+test_name='svcdgs'
+nrg_file=${srcdir}/${test_name}.nrg
 if test -f  $nrg_file ; then
   test_cdinfo "-q --nrg-file $nrg_file $vcd_opt --iso9660" \
-    svcdgs.dump ${srcdir}/svcdgs.right
+    ${test_name}.dump ${srcdir}/${test_name}.right
   RC=$?
-  check_result $RC 'cd-info NRG test 3'
+  check_result $RC "cd-info NRG $test_name"
+else 
+  echo "Don't see NRG file ${nrg_file}. Test skipped."
+  exit $SKIP_TEST_EXITCODE
+fi
+
+test_name='cdda-mcn'
+nrg_file=${srcdir}/${test_name}.nrg
+if test -f  $nrg_file ; then
+  test_cdinfo "-q --nrg-file $nrg_file --no-cddb" \
+    ${test_name}.dump ${srcdir}/${test_name}.right
+  RC=$?
+  check_result $RC "cd-info NRG $test_name"
   
   exit $RC
 else 
